"""Consistent table naming

Revision ID: 45b5ec1ed88b
Revises: 6565544dfb94
Create Date: 2022-04-03 08:59:48.992053

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "45b5ec1ed88b"
down_revision = "6565544dfb94"
branch_labels = None
depends_on = None


def upgrade():
    op.rename_table("record_comments", "record_comment")
    op.execute(sa.text("ALTER SEQUENCE record_comments_id_seq RENAME TO record_comment_id_seq"))
    op.execute(sa.text("ALTER INDEX record_comments_pkey RENAME TO record_comment_pkey"))
    op.execute(
        sa.text(
            "ALTER TABLE record_comment RENAME CONSTRAINT record_comments_record_id_fkey TO record_comment_record_id_fkey"
        )
    )

    op.rename_table("compute_manager_logs", "compute_manager_log")
    op.execute(sa.text("ALTER SEQUENCE compute_manager_logs_id_seq RENAME TO compute_manager_log_id_seq"))
    op.execute(sa.text("ALTER INDEX ix_compute_manager_logs_timestamp RENAME TO ix_compute_manager_log_timestamp"))
    op.execute(sa.text("ALTER INDEX compute_manager_logs_pkey RENAME TO compute_manager_log_pkey"))
    op.execute(
        sa.text(
            "ALTER TABLE compute_manager_log RENAME CONSTRAINT compute_manager_logs_manager_id_fkey TO compute_manager_log_manager_id_fkey"
        )
    )

    op.rename_table("reaction_components", "reaction_component")
    op.execute(sa.text("ALTER INDEX reaction_components_pkey RENAME TO reaction_component_pkey"))
    op.execute(
        sa.text(
            "ALTER TABLE reaction_component RENAME CONSTRAINT reaction_components_molecule_id_fkey TO reaction_component_molecule_id_fkey"
        )
    )
    op.execute(
        sa.text(
            "ALTER TABLE reaction_component RENAME CONSTRAINT reaction_components_singlepoint_id_fkey TO reaction_component_singlepoint_id_fkey"
        )
    )
    op.execute(
        sa.text(
            "ALTER TABLE reaction_component RENAME CONSTRAINT reaction_components_reaction_id_fkey TO reaction_component_reaction_id_fkey"
        )
    )
    op.execute(
        sa.text(
            "ALTER TABLE reaction_component RENAME CONSTRAINT reaction_components_reaction_id_molecule_id_fkey TO reaction_component_reaction_id_molecule_id_fkey"
        )
    )

    op.rename_table("reaction_stoichiometries", "reaction_stoichiometry")
    op.execute(sa.text("ALTER INDEX reaction_stoichiometries_pkey RENAME TO reaction_stoichiometry_pkey"))
    op.execute(
        sa.text(
            "ALTER TABLE reaction_stoichiometry RENAME CONSTRAINT reaction_stoichiometries_molecule_id_fkey TO reaction_stoichiometry_molecule_id_fkey"
        )
    )
    op.execute(
        sa.text(
            "ALTER TABLE reaction_stoichiometry RENAME CONSTRAINT reaction_stoichiometries_reaction_id_fkey TO reaction_stoichiometry_reaction_id_fkey"
        )
    )

    op.rename_table("service_dependencies", "service_dependency")
    op.execute(sa.text("ALTER SEQUENCE service_dependencies_id_seq RENAME TO service_dependency_id_seq"))
    op.execute(sa.text("ALTER INDEX service_dependencies_pkey RENAME TO service_dependency_pkey"))
    op.execute(
        sa.text(
            "ALTER TABLE service_dependency RENAME CONSTRAINT service_dependencies_record_id_fkey TO service_dependency_record_id_fkey"
        )
    )
    op.execute(
        sa.text(
            "ALTER TABLE service_dependency RENAME CONSTRAINT service_dependencies_service_id_fkey TO service_dependency_service_id_fkey"
        )
    )
    op.execute(
        sa.text("ALTER TABLE service_dependency RENAME CONSTRAINT ux_service_dependencies TO ux_service_dependency")
    )

    op.rename_table("gridoptimization_optimizations", "gridoptimization_optimization")
    op.execute(sa.text("ALTER INDEX gridoptimization_optimizations_pkey RENAME TO gridoptimization_optimization_pkey"))
    op.execute(
        sa.text(
            "ALTER TABLE gridoptimization_optimization RENAME CONSTRAINT gridoptimization_optimizations_gridoptimization_id_fkey TO gridoptimization_optimization_gridoptimization_id_fkey"
        )
    )
    op.execute(
        sa.text(
            "ALTER TABLE gridoptimization_optimization RENAME CONSTRAINT gridoptimization_optimizations_optimization_id_fkey TO gridoptimization_optimization_optimization_id_fkey"
        )
    )

    op.rename_table("torsiondrive_optimizations", "torsiondrive_optimization")
    op.execute(sa.text("ALTER INDEX torsiondrive_optimizations_pkey RENAME TO torsiondrive_optimization_pkey"))
    op.execute(
        sa.text(
            "ALTER TABLE torsiondrive_optimization RENAME CONSTRAINT torsiondrive_optimizations_optimization_id_fkey TO torsiondrive_optimization_optimization_id_fkey"
        )
    )
    op.execute(
        sa.text(
            "ALTER TABLE torsiondrive_optimization RENAME CONSTRAINT torsiondrive_optimizations_torsiondrive_id_fkey TO torsiondrive_optimization_torsiondrive_id_fkey"
        )
    )

    op.rename_table("torsiondrive_initial_molecules", "torsiondrive_initial_molecule")
    op.execute(sa.text("ALTER INDEX torsiondrive_initial_molecules_pkey RENAME TO torsiondrive_initial_molecule_pkey"))
    op.execute(
        sa.text(
            "ALTER TABLE torsiondrive_initial_molecule RENAME CONSTRAINT torsiondrive_initial_molecules_molecule_id_fkey TO torsiondrive_initial_molecule_molecule_id_fkey"
        )
    )
    op.execute(
        sa.text(
            "ALTER TABLE torsiondrive_initial_molecule RENAME CONSTRAINT torsiondrive_initial_molecules_torsiondrive_id_fkey TO torsiondrive_initial_molecule_torsiondrive_id_fkey"
        )
    )

    # Now renaming stuff in the server logs
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'compute_manager_logs', 'compute_manager_log')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'record_comments', 'record_comment')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'service_dependencies', 'service_dependency')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'torsiondrive_optimizations', 'torsiondrive_optimization')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'torsiondrive_initial_molecules', 'torsiondrive_initial_molecule')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'gridoptimization_optimizations', 'gridoptimization_optimization')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'reaction_components', 'reaction_component')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'reaction_stoichiometries', 'reaction_stoichiometry')::json"
        )
    )

    # Others I missed before
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'kv_store', 'output_store')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, '\"dataset\"', '\"singlepoint_dataset\"')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, '\"dataset_entry\"', '\"singlepoint_dataset_entry\"')::json"
        )
    )


def downgrade():
    raise RuntimeError("Cannot downgrade")



"""optimization dataset

Revision ID: bf4b379a6ce4
Revises: 4485daecc21b
Create Date: 2022-02-18 10:31:50.528949

"""

import json
import os
import sys

import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import postgresql
from sqlalchemy.orm.session import Session
from sqlalchemy.sql import table, column

sys.path.insert(1, os.path.dirname(os.path.abspath(__file__)))
from migration_helpers.v0_50_helpers import get_empty_keywords_id, add_opt_spec, add_qc_spec

# revision identifiers, used by Alembic.
revision = "bf4b379a6ce4"
down_revision = "4485daecc21b"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "optimization_dataset",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["id"], ["collection.id"], ondelete="cascade"),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "optimization_dataset_entry",
        sa.Column("dataset_id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("comment", sa.String(), nullable=True),
        sa.Column("initial_molecule_id", sa.Integer(), nullable=False),
        sa.Column("additional_keywords", postgresql.JSONB(astext_type=sa.Text()), nullable=True),
        sa.Column("attributes", postgresql.JSONB(astext_type=sa.Text()), nullable=True),
        sa.ForeignKeyConstraint(["dataset_id"], ["optimization_dataset.id"], ondelete="cascade"),
        sa.ForeignKeyConstraint(
            ["initial_molecule_id"],
            ["molecule.id"],
        ),
        sa.PrimaryKeyConstraint("dataset_id", "name"),
    )
    op.create_index(
        "ix_optimization_dataset_entry_dataset_id", "optimization_dataset_entry", ["dataset_id"], unique=False
    )
    op.create_index(
        "ix_optimization_dataset_entry_initial_molecule_id",
        "optimization_dataset_entry",
        ["initial_molecule_id"],
        unique=False,
    )
    op.create_index("ix_optimization_dataset_entry_name", "optimization_dataset_entry", ["name"], unique=False)
    op.create_table(
        "optimization_dataset_specification",
        sa.Column("dataset_id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("description", sa.String(), nullable=True),
        sa.Column("specification_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["dataset_id"], ["optimization_dataset.id"], ondelete="cascade"),
        sa.ForeignKeyConstraint(
            ["specification_id"],
            ["optimization_specification.id"],
        ),
        sa.PrimaryKeyConstraint("dataset_id", "name"),
    )
    op.create_index(
        "ix_optimization_dataset_specification_dataset_id",
        "optimization_dataset_specification",
        ["dataset_id"],
        unique=False,
    )
    op.create_index(
        "ix_optimization_dataset_specification_name", "optimization_dataset_specification", ["name"], unique=False
    )
    op.create_index(
        "ix_optimization_dataset_specification_specification_id",
        "optimization_dataset_specification",
        ["specification_id"],
        unique=False,
    )
    op.create_table(
        "optimization_dataset_record",
        sa.Column("dataset_id", sa.Integer(), nullable=False),
        sa.Column("entry_name", sa.String(), nullable=False),
        sa.Column("specification_name", sa.String(), nullable=False),
        sa.Column("record_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["dataset_id", "entry_name"],
            ["optimization_dataset_entry.dataset_id", "optimization_dataset_entry.name"],
            onupdate="cascade",
            ondelete="cascade",
        ),
        sa.ForeignKeyConstraint(
            ["dataset_id", "specification_name"],
            ["optimization_dataset_specification.dataset_id", "optimization_dataset_specification.name"],
            onupdate="cascade",
            ondelete="cascade",
        ),
        sa.ForeignKeyConstraint(["dataset_id"], ["optimization_dataset.id"], ondelete="cascade"),
        sa.ForeignKeyConstraint(
            ["record_id"],
            ["optimization_record.id"],
        ),
        sa.PrimaryKeyConstraint("dataset_id", "entry_name", "specification_name"),
        sa.UniqueConstraint(
            "dataset_id", "entry_name", "specification_name", name="ux_optimization_dataset_record_unique"
        ),
    )
    op.create_index(
        "ix_optimization_dataset_record_record_id", "optimization_dataset_record", ["record_id"], unique=False
    )

    # Add default tag/priority to base collection class
    op.add_column("collection", sa.Column("default_tag", sa.String(), nullable=True))
    op.add_column("collection", sa.Column("default_priority", sa.Integer(), nullable=True))

    op.execute(sa.text("UPDATE collection SET default_tag = '*' where default_tag IS NULL"))
    op.execute(sa.text("UPDATE collection SET default_priority = 1 where default_priority IS NULL"))

    op.alter_column("collection", "default_tag", nullable=False)
    op.alter_column("collection", "default_priority", nullable=False)

    ######################
    # MIGRATION
    ######################
    conn = op.get_bind()

    # For all collections
    conn.execute(
        sa.text(
            """UPDATE collection
        SET provenance = NULL
        WHERE provenance::jsonb = 'null'::jsonb
        """
        )
    )

    # Just for optimization
    op.execute(
        sa.text("UPDATE collection SET collection_type = 'optimization' where collection = 'optimizationdataset'")
    )
    op.execute(sa.text("UPDATE collection SET collection = 'optimization' where collection = 'optimizationdataset'"))

    # Temporary ORM
    collection_table = table(
        "collection",
        column("id", sa.Integer),
        column("collection", sa.String),
        column("extra", sa.JSON),
    )

    session = Session(conn)
    collections = session.query(collection_table).where(collection_table.c.collection == "optimization").all()
    for col in collections:
        ext = col.extra
        col_specs = ext.pop("specs")
        col_record = ext.pop("records")

        # Add the dataset to the separate optimziation dataset table
        conn.execute(sa.text("INSERT INTO optimization_dataset (id) VALUES (:colid)"), parameters={"colid": col.id})

        # Specifications
        # Empty keywords
        empty_kw_id = get_empty_keywords_id(conn)

        #########################
        # SPECIFICATIONS
        #########################
        for spec_name, spec in col_specs.items():
            # Protocols for qc_spec were always ignored. So set them with the default
            opt_spec = spec["optimization_spec"]
            qc_spec = spec["qc_spec"]
            qc_spec["driver"] = "deferred"

            if qc_spec.get("keywords", None) is None:
                qc_spec["keywords"] = empty_kw_id

            qc_spec_id = add_qc_spec(
                conn,
                qc_spec["program"],
                qc_spec["driver"],
                qc_spec["method"],
                qc_spec["basis"],
                qc_spec["keywords"],
                {},
            )

            # Optimization spec
            if opt_spec.get("protocols", None) is None:
                opt_spec["protocols"] = {}
            if opt_spec.get("keywords", None) is None:
                opt_spec["keywords"] = {}

            opt_spec_id = add_opt_spec(
                conn, qc_spec_id, opt_spec["program"], opt_spec["keywords"], opt_spec["protocols"]
            )

            # Now add to the dataset spec
            conn.execute(
                sa.text(
                    """
                       INSERT INTO optimization_dataset_specification (dataset_id, name, description, specification_id)
                       VALUES (:col_id, :spec_name, :spec_desc, :opt_spec_id)
                    """
                ),
                parameters=dict(
                    col_id=col.id, spec_name=spec["name"], spec_desc=spec["description"], opt_spec_id=opt_spec_id
                ),
            )

        ####################
        # ENTRIES/RECORDS
        ####################
        conn.execute(
            sa.text(
                """
                   INSERT INTO optimization_dataset_entry (dataset_id, name, initial_molecule_id, additional_keywords, attributes)
                   SELECT :col_id, j.key, m.id, (j.value->>'additional_keywords')::jsonb, (j.value->>'attributes')::jsonb
                   FROM collection, json_each(extra->'records') AS j
                   INNER JOIN molecule m ON m.id = (j.value->>'initial_molecule')::integer
                   WHERE collection.id = :col_id
                """
            ),
            parameters=dict(col_id=col.id),
        )

        for ent_name, ent in col_record.items():
            for spec_name, record_id in ent["object_map"].items():
                conn.execute(
                    sa.text(
                        """
                           INSERT INTO optimization_dataset_record (dataset_id, entry_name, specification_name, record_id)
                           SELECT :col_id, :ent_name, :spec_name, r.id
                           FROM optimization_record r
                           WHERE r.id = :record_id
                           """
                    ),
                    parameters=dict(
                        col_id=col.id,
                        ent_name=ent_name,
                        spec_name=spec_name,
                        record_id=record_id,
                    ),
                )

        # Update the collection extra, with the removed fields
        ext.pop("history", None)
        conn.execute(
            sa.text("UPDATE collection SET extra = (:extra)::json WHERE id = :col_id"),
            parameters=dict(
                col_id=col.id,
                extra=json.dumps(ext),
            ),
        )

    # ### end Alembic commands ###


def downgrade():
    raise RuntimeError("Cannot downgrade")



"""Add indices to timestamps in base_record

Revision ID: a2e52a5e90b6
Revises: b03545e6292a
Create Date: 2023-06-14 13:55:27.574071

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "a2e52a5e90b6"
down_revision = "b03545e6292a"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index("ix_base_record_created_on", "base_record", ["created_on"], unique=False, postgresql_using="brin")
    op.create_index("ix_base_record_modified_on", "base_record", ["modified_on"], unique=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("ix_base_record_modified_on", table_name="base_record")
    op.drop_index("ix_base_record_created_on", table_name="base_record", postgresql_using="brin")
    # ### end Alembic commands ###



"""Migrate properties 1

Revision ID: 48d1c43c27e0
Revises: 3cc95f9dc02c
Create Date: 2022-11-10 10:44:17.844144

"""

import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "48d1c43c27e0"
down_revision = "3cc95f9dc02c"
branch_labels = None
depends_on = None


def upgrade():
    # New temporary JSONB columns
    # We need to slowly migrate to there
    op.add_column("base_record", sa.Column("new_extras", postgresql.JSONB, nullable=True))
    op.add_column("base_record", sa.Column("new_properties", postgresql.JSONB, nullable=True))


def downgrade():
    raise RuntimeError("CANNOT DOWNGRADE")



"""Migrate services

Revision ID: 651954cbdec6
Revises: 98aea37d208d
Create Date: 2021-07-13 09:53:23.243644

"""

import json

import numpy
import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import postgresql
from sqlalchemy.orm.session import Session
from sqlalchemy.sql import table, column

from qcfractal.db_socket.column_types import MsgpackExt, PlainMsgpackExt

# revision identifiers, used by Alembic.
revision = "651954cbdec6"
down_revision = "98aea37d208d"
branch_labels = None
depends_on = None


def convert_numpy(d):
    if isinstance(d, dict):
        for k, v in d.items():
            d[k] = convert_numpy(v)
        return d
    elif isinstance(d, numpy.ndarray):
        return d.tolist()
    elif isinstance(d, list):
        return [convert_numpy(x) for x in d]
    else:
        return d


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###

    # Create the auxiliary table for tasks associated with services
    op.create_table(
        "service_queue_tasks",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("service_id", sa.Integer(), nullable=False),
        sa.Column("procedure_id", sa.Integer(), nullable=False),
        sa.Column("extras", sa.JSON(), nullable=True),
        sa.ForeignKeyConstraint(["procedure_id"], ["base_result.id"], ondelete="cascade"),
        sa.ForeignKeyConstraint(["service_id"], ["service_queue.id"], ondelete="cascade"),
        sa.PrimaryKeyConstraint("id"),
    )

    # Handle the service state
    # Add the column and then migrate what was in extra
    op.add_column("service_queue", sa.Column("service_state", PlainMsgpackExt(), nullable=True))

    # Temporary ORM
    service_tasks_table = table(
        "service_queue_tasks",
        column("service_id", sa.Integer),
        column("procedure_id", sa.Integer),
        column("extras", sa.JSON),
    )

    service_table = table(
        "service_queue",
        column("extra", MsgpackExt),
        column("service_state", PlainMsgpackExt),
        column("id", sa.Integer),
        column("tag", sa.String),
        column("priority", sa.Integer),
        column("procedure_id", sa.Integer),
    )

    kvstore_table = table("kv_store", column("id", sa.Integer), column("value", sa.JSON))

    baseresult_table = table(
        "base_result", column("id", sa.Integer), column("stdout", sa.Integer), column("error", sa.Integer)
    )

    bind = op.get_bind()
    session = Session(bind=bind)

    services = session.query(service_table).all()

    for service in services:
        if "torsiondrive_state" in service.extra:
            # We have a torsiondrive
            service_state = {}
            service_state["torsiondrive_state"] = convert_numpy(service.extra["torsiondrive_state"])

            service_state["molecule_template"] = service.extra["molecule_template"]
            service_state["dihedral_template"] = service.extra["dihedral_template"]

            old_opt_template = json.loads(service.extra["optimization_template"])
            old_opt_template["meta"]["priority"] = service.priority

            # We don't use "meta" anymore. Just store the value
            service_state["optimization_template"] = json.dumps(old_opt_template["meta"])

            # Pull out the maps of subtasks, and convert them to the new association table
            for td_api_key, task_keys in service.extra["task_map"].items():
                for task_key in task_keys:
                    task_id = service.extra["task_manager"]["required_tasks"][task_key]
                    position = task_key.split("-")[-1]

                    insert_values = {
                        "service_id": int(service.id),
                        "procedure_id": int(task_id),
                        "extras": {"td_api_key": td_api_key, "position": position},
                    }
                    session.execute(service_tasks_table.insert().values(insert_values))

        else:
            # We have a grid optimization
            service_state = {}
            service_state["iteration"] = service.extra["iteration"]
            service_state["dimensions"] = service.extra["dimensions"]
            service_state["complete"] = service.extra["complete"]
            service_state["constraint_template"] = service.extra["constraint_template"]

            old_opt_template = json.loads(service.extra["optimization_template"])
            old_opt_template["meta"]["priority"] = service.priority

            # We don't use "meta" anymore. Just store the value
            service_state["optimization_template"] = json.dumps(old_opt_template["meta"])

            # Pull out the maps of subtasks, and convert them to the new association table
            for key, task_id in service.extra["task_manager"]["required_tasks"].items():
                insert_values = {"service_id": int(service.id), "procedure_id": int(task_id), "extras": {"key": key}}

                session.execute(service_tasks_table.insert().values(insert_values))

        # Set these to be the same priority as the tasks. I don't believe the priority or tag
        # of the service themselves was ever modifiable
        update_values = {
            "service_state": service_state,
            "tag": service.extra["task_tag"],
            "priority": service.extra["task_priority"],
        }

        session.execute(service_table.update().values(update_values).where(service_table.c.id == service.id))

        # Handle outputs and errors
        stdout = service.extra.get("stdout")
        if stdout:
            stdout_id = session.execute(kvstore_table.insert().values({"value": stdout}).returning(sa.column("id")))
            stdout_id = list(stdout_id)
            assert len(stdout_id) == 1 and len(stdout_id[0]) == 1
            stmt = (
                baseresult_table.update()
                .values({"stdout": stdout_id[0][0]})
                .where(baseresult_table.c.id == service.procedure_id)
            )
            session.execute(stmt)

        error = service.extra.get("error")
        if error:
            error_id = session.execute(kvstore_table.insert().values({"value": error}).returning(sa.column("id")))
            error_id = list(error_id)
            assert len(error_id) == 1 and len(error_id[0]) == 1
            stmt = (
                baseresult_table.update()
                .values({"error": error_id[0][0]})
                .where(baseresult_table.c.id == service.procedure_id)
            )
            session.execute(stmt)

        session.flush()

    # Now drop the unneeded stuff
    op.drop_index("ix_grid_optmization_program", table_name="grid_optimization_procedure")
    op.drop_column("grid_optimization_procedure", "program")
    op.drop_index("ix_service_queue_hash_index", table_name="service_queue")
    op.drop_column("service_queue", "modified_on")
    op.drop_column("service_queue", "extra")
    op.drop_column("service_queue", "hash_index")
    op.drop_index("ix_torsion_drive_program", table_name="torsiondrive_procedure")
    op.drop_column("torsiondrive_procedure", "program")
    # ### end Alembic commands ###


def downgrade():
    raise RuntimeError("Cannot downgrade")

    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "torsiondrive_procedure", sa.Column("program", sa.VARCHAR(length=100), autoincrement=False, nullable=False)
    )
    op.create_index("ix_torsion_drive_program", "torsiondrive_procedure", ["program"], unique=False)
    op.add_column("service_queue", sa.Column("hash_index", sa.VARCHAR(), autoincrement=False, nullable=False))
    op.add_column("service_queue", sa.Column("extra", postgresql.BYTEA(), autoincrement=False, nullable=True))
    op.add_column("service_queue", sa.Column("modified_on", postgresql.TIMESTAMP(), autoincrement=False, nullable=True))
    op.create_index("ix_service_queue_hash_index", "service_queue", ["hash_index"], unique=False)
    op.drop_column("service_queue", "service_state")
    op.add_column(
        "grid_optimization_procedure", sa.Column("program", sa.VARCHAR(length=100), autoincrement=False, nullable=False)
    )
    op.create_index("ix_grid_optmization_program", "grid_optimization_procedure", ["program"], unique=False)
    op.drop_table("service_queue_tasks")
    # ### end Alembic commands ###



"""remove gzip and bzip, add zstd

Revision ID: 48f4d60735cf
Revises: 8fc81746daa6
Create Date: 2022-09-05 10:11:48.344125

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "48f4d60735cf"
down_revision = "8fc81746daa6"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.execute("ALTER TYPE compressionenum RENAME TO compressionenum_old")
    new_enum = sa.Enum("none", "lzma", "zstd", name="compressionenum")
    new_enum.create(op.get_bind(), checkfirst=True)

    op.add_column("output_store", sa.Column("new_compression", new_enum))
    op.add_column("native_file", sa.Column("new_compression", new_enum))

    op.execute("UPDATE output_store SET new_compression = compression::text::compressionenum")
    op.execute("UPDATE native_file SET new_compression = compression::text::compressionenum")

    op.drop_column("output_store", "compression")
    op.alter_column("output_store", "new_compression", new_column_name="compression")

    op.drop_column("native_file", "compression")
    op.alter_column("native_file", "new_compression", new_column_name="compression", nullable=False)

    op.execute("DROP TYPE compressionenum_old")

    # ### end Alembic commands ###


def downgrade():
    raise RuntimeError("Cannot downgrade")



"""Some columns not nullable in access log

Revision ID: 84e421577a1a
Revises: e645774b4c62
Create Date: 2023-03-17 13:15:51.493867

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "84e421577a1a"
down_revision = "e645774b4c62"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###

    op.execute(sa.text("UPDATE access_log SET request_duration = 0.0 WHERE request_duration IS NULL"))
    op.execute(sa.text("UPDATE access_log SET request_bytes = 0.0 WHERE request_bytes IS NULL"))
    op.execute(sa.text("UPDATE access_log SET response_bytes = 0.0 WHERE response_bytes IS NULL"))

    op.alter_column("access_log", "request_duration", existing_type=sa.DOUBLE_PRECISION(precision=53), nullable=False)
    op.alter_column("access_log", "request_bytes", existing_type=sa.BIGINT(), nullable=False)
    op.alter_column("access_log", "response_bytes", existing_type=sa.BIGINT(), nullable=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("access_log", "response_bytes", existing_type=sa.BIGINT(), nullable=True)
    op.alter_column("access_log", "request_bytes", existing_type=sa.BIGINT(), nullable=True)
    op.alter_column("access_log", "request_duration", existing_type=sa.DOUBLE_PRECISION(precision=53), nullable=True)
    # ### end Alembic commands ###



"""rename kv_store table

Revision ID: 2a334994fcb3
Revises: d36e2ec09f27
Create Date: 2021-10-10 11:47:58.500027

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "2a334994fcb3"
down_revision = "d36e2ec09f27"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.rename_table("kv_store", "output_store")
    op.execute(sa.text("ALTER SEQUENCE kv_store_id_seq RENAME TO output_store_id_seq"))
    op.execute(sa.text("ALTER INDEX kv_store_pkey RENAME TO output_store_pkey"))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.rename_table("output_store", "kv_store")
    op.execute(sa.text("ALTER SEQUENCE output_store_id_seq RENAME TO kv_store_id_seq"))
    op.execute(sa.text("ALTER INDEX output_store_pkey RENAME TO kv_store_pkey"))
    # ### end Alembic commands ###



"""singlepoint dataset

Revision ID: efaea8a3b4c0
Revises: bf4b379a6ce4
Create Date: 2022-02-21 10:19:54.041035

"""

import os
import sys

import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import postgresql
from sqlalchemy.orm.session import Session
from sqlalchemy.sql import table, column

sys.path.insert(1, os.path.dirname(os.path.abspath(__file__)))
from migration_helpers.v0_50_helpers import get_empty_keywords_id, add_qc_spec

# revision identifiers, used by Alembic.
revision = "efaea8a3b4c0"
down_revision = "bf4b379a6ce4"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "singlepoint_dataset_specification",
        sa.Column("dataset_id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("description", sa.String(), nullable=True),
        sa.Column("specification_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["dataset_id"], ["dataset.id"], ondelete="cascade"),
        sa.ForeignKeyConstraint(
            ["specification_id"],
            ["qc_specification.id"],
        ),
        sa.PrimaryKeyConstraint("dataset_id", "name"),
    )
    op.create_index(
        "ix_singlepoint_dataset_specification_dataset_id",
        "singlepoint_dataset_specification",
        ["dataset_id"],
        unique=False,
    )
    op.create_index(
        "ix_singlepoint_dataset_specification_name", "singlepoint_dataset_specification", ["name"], unique=False
    )
    op.create_index(
        "ix_singlepoint_dataset_specification_specification_id",
        "singlepoint_dataset_specification",
        ["specification_id"],
        unique=False,
    )

    op.create_table(
        "singlepoint_dataset_record",
        sa.Column("dataset_id", sa.Integer(), nullable=False),
        sa.Column("entry_name", sa.String(), nullable=False),
        sa.Column("specification_name", sa.String(), nullable=False),
        sa.Column("record_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["dataset_id", "entry_name"],
            ["dataset_entry.dataset_id", "dataset_entry.name"],
            onupdate="cascade",
            ondelete="cascade",
        ),
        sa.ForeignKeyConstraint(
            ["dataset_id", "specification_name"],
            ["singlepoint_dataset_specification.dataset_id", "singlepoint_dataset_specification.name"],
            onupdate="cascade",
            ondelete="cascade",
        ),
        sa.ForeignKeyConstraint(["dataset_id"], ["dataset.id"], ondelete="cascade"),
        sa.ForeignKeyConstraint(
            ["record_id"],
            ["singlepoint_record.id"],
        ),
        sa.PrimaryKeyConstraint("dataset_id", "entry_name", "specification_name"),
        sa.UniqueConstraint(
            "dataset_id", "entry_name", "specification_name", name="ux_singlepoint_dataset_record_unique"
        ),
    )
    op.create_index(
        "ix_singlepoint_dataset_record_record_id", "singlepoint_dataset_record", ["record_id"], unique=False
    )

    # Rename dataset table pkey
    op.execute(sa.text("ALTER INDEX dataset_pkey RENAME TO singlepoint_dataset_pkey"))

    op.add_column(
        "dataset_entry", sa.Column("additional_keywords", postgresql.JSONB(astext_type=sa.Text()), nullable=True)
    )
    op.add_column("dataset_entry", sa.Column("attributes", postgresql.JSONB(astext_type=sa.Text()), nullable=True))
    op.create_foreign_key(
        "singlepoint_dataset_entry_dataset_id_fkey",
        "dataset_entry",
        "dataset",
        ["dataset_id"],
        ["id"],
        ondelete="CASCADE",
    )

    op.create_index("ix_singlepoint_dataset_entry_dataset_id", "dataset_entry", ["dataset_id"], unique=False)

    op.create_index("ix_singlepoint_dataset_entry_molecule_id", "dataset_entry", ["molecule_id"], unique=False)

    op.create_index("ix_singlepoint_dataset_entry_name", "dataset_entry", ["name"], unique=False)

    op.create_foreign_key(
        "singlepoint_dataset_entry_molecule_id_fkey", "dataset_entry", "molecule", ["molecule_id"], ["id"]
    )
    op.drop_constraint("dataset_entry_molecule_id_fkey", "dataset_entry", type_="foreignkey")
    op.drop_constraint("dataset_entry_dataset_id_fkey", "dataset_entry", type_="foreignkey")

    # dataset entry pkey and fkey
    op.drop_constraint("dataset_id_fkey", "dataset", type_="foreignkey")
    op.create_foreign_key("singlepoint_dataset_id_fkey", "dataset", "collection", ["id"], ["id"], ondelete="CASCADE")
    op.execute(sa.text("ALTER INDEX dataset_entry_pkey RENAME TO singlepoint_dataset_entry_pkey"))

    ##############################
    # DATA MIGRATION
    ##############################
    conn = op.get_bind()

    op.execute(sa.text("UPDATE collection SET collection_type = 'singlepoint' where collection = 'dataset'"))
    op.execute(sa.text("UPDATE collection SET collection = 'singlepoint' where collection = 'dataset'"))

    # Temporary ORM
    dataset_table = table(
        "dataset",
        column("id", sa.Integer),
        column("history", sa.JSON),
        column("history_keys", sa.JSON),
        column("alias_keywords", sa.JSON),
    )

    session = Session(conn)
    datasets = session.query(dataset_table).all()

    empty_kw = get_empty_keywords_id(conn)

    for ds in datasets:
        spec_idx = 1
        for h in ds.history:
            spec = dict(zip(ds.history_keys, h))

            # keywords should be in alias_keywords, except for dftd3 directly run through the
            # composition planner......
            try:
                if spec["keywords"] is None:
                    kw = None
                else:
                    kw = ds.alias_keywords[spec["program"]][spec["keywords"]]
            except KeyError:
                if spec["program"] == "dftd3":
                    kw = empty_kw
                else:
                    raise RuntimeError(f"Missing entry from alias_keywords: {spec['program']}, {spec['keywords']}")

            if kw is None:
                kw = empty_kw
            if spec["basis"] is None:
                spec["basis"] = ""

            # What specifications match? Protocols wasn't taken into account
            res = conn.execute(
                sa.text(
                    """
                    SELECT q.id FROM qc_specification q
                    WHERE q.program = :program
                    AND q.driver = :driver
                    AND q.method = :method
                    AND q.basis = :basis
                    AND q.keywords_id = :keywords_id
                    """
                ),
                parameters=dict(
                    col_id=ds.id,
                    program=spec["program"],
                    driver=spec["driver"],
                    method=spec["method"],
                    basis="" if spec["basis"] is None else spec["basis"],
                    keywords_id=kw,
                ),
            )

            spec_ids = res.scalars().all()

            # If not already existing, add it
            if len(spec_ids) == 0:
                sid = add_qc_spec(
                    conn,
                    spec["program"],
                    spec["driver"],
                    spec["method"],
                    spec["basis"],
                    kw,
                    spec.get("protocols", {}),
                )

                spec_ids = [sid]

            # Add these to the spec table
            for sid in spec_ids:
                spec_name = f"spec_{spec_idx}"
                spec_idx += 1

                conn.execute(
                    sa.text(
                        """INSERT INTO singlepoint_dataset_specification (dataset_id, name, description, specification_id)
                                      VALUES (:col_id, :spec_name, :spec_desc, :qc_spec_id)"""
                    ),
                    parameters=dict(col_id=ds.id, spec_name=spec_name, spec_desc="", qc_spec_id=sid),
                )

                # Now what records match these specs
                conn.execute(
                    sa.text(
                        """
                        INSERT INTO singlepoint_dataset_record (dataset_id, entry_name, specification_name, record_id)
                        SELECT e.dataset_id, e.name, s.name, r.id
                        FROM dataset_entry e, singlepoint_dataset_specification s, singlepoint_record r 
                        WHERE e.dataset_id = :col_id
                        AND s.dataset_id = :col_id
                        AND s.specification_id = r.specification_id
                        AND r.specification_id = :qc_spec_id
                        AND e.molecule_id = r.molecule_id
                        ON CONFLICT DO NOTHING
                        """
                    ),
                    parameters=dict(col_id=ds.id, qc_spec_id=sid),
                )

    # Drop old dataset columns
    op.drop_column("dataset", "default_benchmark")
    op.drop_column("dataset", "default_program")
    op.drop_column("dataset", "default_driver")
    op.drop_column("dataset", "default_keywords")
    op.drop_column("dataset", "default_units")
    op.drop_column("dataset", "history")
    op.drop_column("dataset", "history_keys")
    op.drop_column("dataset", "alias_keywords")

    # Finally rename the tables
    op.rename_table("dataset", "singlepoint_dataset")
    op.rename_table("dataset_entry", "singlepoint_dataset_entry")

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    raise RuntimeError("Cannot downgrade")



"""Add native files

Revision ID: 6565544dfb94
Revises: 9a86ebfcfd1b
Create Date: 2022-03-30 21:38:03.647374

"""

import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "6565544dfb94"
down_revision = "9a86ebfcfd1b"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    compression_enum = postgresql.ENUM("none", "gzip", "bzip2", "lzma", name="compressionenum", create_type=False)

    op.create_table(
        "native_file",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("record_id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("compression", compression_enum, nullable=False),
        sa.Column("compression_level", sa.Integer(), nullable=False),
        sa.Column("is_text", sa.Boolean(), nullable=False),
        sa.Column("uncompressed_size", sa.Integer(), nullable=False),
        sa.Column("data", sa.LargeBinary(), nullable=False),
        sa.ForeignKeyConstraint(["record_id"], ["base_record.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("record_id", "name", name="ux_native_file_record_id_name"),
    )
    op.create_index("ix_native_file_record_id", "native_file", ["record_id"], unique=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("ix_native_file_record_id", table_name="native_file")
    op.drop_table("native_file")
    # ### end Alembic commands ###



"""migrate wavefunction table

Revision ID: 05f7b00bfac8
Revises: fd76a1459fa3
Create Date: 2021-10-13 16:59:18.738338

"""

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "05f7b00bfac8"
down_revision = "fd76a1459fa3"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("wavefunction_store", sa.Column("orbitals_a", sa.String(), nullable=True))
    op.add_column("wavefunction_store", sa.Column("orbitals_b", sa.String(), nullable=True))
    op.add_column("wavefunction_store", sa.Column("density_a", sa.String(), nullable=True))
    op.add_column("wavefunction_store", sa.Column("density_b", sa.String(), nullable=True))
    op.add_column("wavefunction_store", sa.Column("fock_a", sa.String(), nullable=True))
    op.add_column("wavefunction_store", sa.Column("fock_b", sa.String(), nullable=True))
    op.add_column("wavefunction_store", sa.Column("eigenvalues_a", sa.String(), nullable=True))
    op.add_column("wavefunction_store", sa.Column("eigenvalues_b", sa.String(), nullable=True))
    op.add_column("wavefunction_store", sa.Column("occupations_a", sa.String(), nullable=True))
    op.add_column("wavefunction_store", sa.Column("occupations_b", sa.String(), nullable=True))
    op.drop_column("wavefunction_store", "extras")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("wavefunction_store", sa.Column("extras", postgresql.BYTEA(), autoincrement=False, nullable=True))
    op.drop_column("wavefunction_store", "occupations_b")
    op.drop_column("wavefunction_store", "occupations_a")
    op.drop_column("wavefunction_store", "eigenvalues_b")
    op.drop_column("wavefunction_store", "eigenvalues_a")
    op.drop_column("wavefunction_store", "fock_b")
    op.drop_column("wavefunction_store", "fock_a")
    op.drop_column("wavefunction_store", "density_b")
    op.drop_column("wavefunction_store", "density_a")
    op.drop_column("wavefunction_store", "orbitals_b")
    op.drop_column("wavefunction_store", "orbitals_a")
    # ### end Alembic commands ###



"""add neb calculations

Revision ID: 8fc81746daa6
Revises: f512f2e7ec3d
Create Date: 2022-08-26 16:49:14.688346

"""

import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "8fc81746daa6"
down_revision = "f512f2e7ec3d"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "neb_dataset",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["id"], ["base_dataset.id"], ondelete="cascade"),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "neb_specification",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("program", sa.String(length=100), nullable=False),
        sa.Column("singlepoint_specification_id", sa.Integer(), nullable=False),
        sa.Column("keywords", postgresql.JSONB(astext_type=sa.Text()), nullable=False),
        sa.ForeignKeyConstraint(
            ["singlepoint_specification_id"],
            ["qc_specification.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("program", "singlepoint_specification_id", "keywords", name="ux_neb_specification_keys"),
    )
    op.create_index("ix_neb_specification_keywords", "neb_specification", ["keywords"], unique=False)
    op.create_index("ix_neb_specification_program", "neb_specification", ["program"], unique=False)
    op.create_index(
        "ix_neb_specification_singlepoint_specification_id",
        "neb_specification",
        ["singlepoint_specification_id"],
        unique=False,
    )
    op.create_table(
        "neb_dataset_entry",
        sa.Column("dataset_id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("comment", sa.String(), nullable=True),
        sa.Column("additional_keywords", postgresql.JSONB(astext_type=sa.Text()), nullable=False),
        sa.Column("attributes", postgresql.JSONB(astext_type=sa.Text()), nullable=False),
        sa.ForeignKeyConstraint(["dataset_id"], ["neb_dataset.id"], ondelete="cascade"),
        sa.PrimaryKeyConstraint("dataset_id", "name"),
    )
    op.create_index("ix_neb_dataset_entry_dataset_id", "neb_dataset_entry", ["dataset_id"], unique=False)
    op.create_index("ix_neb_dataset_entry_name", "neb_dataset_entry", ["name"], unique=False)
    op.create_table(
        "neb_dataset_specification",
        sa.Column("dataset_id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("description", sa.String(), nullable=True),
        sa.Column("specification_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["dataset_id"], ["neb_dataset.id"], ondelete="cascade"),
        sa.ForeignKeyConstraint(
            ["specification_id"],
            ["neb_specification.id"],
        ),
        sa.PrimaryKeyConstraint("dataset_id", "name"),
    )
    op.create_index(
        "ix_neb_dataset_specification_dataset_id", "neb_dataset_specification", ["dataset_id"], unique=False
    )
    op.create_index("ix_neb_dataset_specification_name", "neb_dataset_specification", ["name"], unique=False)
    op.create_index(
        "ix_neb_dataset_specification_specification_id", "neb_dataset_specification", ["specification_id"], unique=False
    )
    op.create_table(
        "neb_record",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("specification_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["id"], ["base_record.id"], ondelete="cascade"),
        sa.ForeignKeyConstraint(
            ["specification_id"],
            ["neb_specification.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "neb_dataset_molecule",
        sa.Column("dataset_id", sa.Integer(), nullable=False),
        sa.Column("entry_name", sa.String(), nullable=False),
        sa.Column("molecule_id", sa.Integer(), nullable=False),
        sa.Column("position", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["dataset_id", "entry_name"],
            ["neb_dataset_entry.dataset_id", "neb_dataset_entry.name"],
            onupdate="cascade",
            ondelete="cascade",
        ),
        sa.ForeignKeyConstraint(
            ["molecule_id"],
            ["molecule.id"],
        ),
        sa.PrimaryKeyConstraint("dataset_id", "entry_name", "molecule_id", "position"),
    )
    op.create_index("ix_neb_dataset_molecule_dataset_id", "neb_dataset_molecule", ["dataset_id"], unique=False)
    op.create_index("ix_neb_dataset_molecule_entry_name", "neb_dataset_molecule", ["entry_name"], unique=False)
    op.create_index("ix_neb_dataset_molecule_molecule_id", "neb_dataset_molecule", ["molecule_id"], unique=False)
    op.create_table(
        "neb_dataset_record",
        sa.Column("dataset_id", sa.Integer(), nullable=False),
        sa.Column("entry_name", sa.String(), nullable=False),
        sa.Column("specification_name", sa.String(), nullable=False),
        sa.Column("record_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["dataset_id", "entry_name"],
            ["neb_dataset_entry.dataset_id", "neb_dataset_entry.name"],
            onupdate="cascade",
            ondelete="cascade",
        ),
        sa.ForeignKeyConstraint(
            ["dataset_id", "specification_name"],
            ["neb_dataset_specification.dataset_id", "neb_dataset_specification.name"],
            onupdate="cascade",
            ondelete="cascade",
        ),
        sa.ForeignKeyConstraint(["dataset_id"], ["neb_dataset.id"], ondelete="cascade"),
        sa.ForeignKeyConstraint(
            ["record_id"],
            ["neb_record.id"],
        ),
        sa.PrimaryKeyConstraint("dataset_id", "entry_name", "specification_name"),
        sa.UniqueConstraint("dataset_id", "entry_name", "specification_name", name="ux_neb_dataset_record_unique"),
    )
    op.create_index("ix_neb_dataset_record_record_id", "neb_dataset_record", ["record_id"], unique=False)
    op.create_table(
        "neb_initialchain",
        sa.Column("neb_id", sa.Integer(), nullable=False),
        sa.Column("molecule_id", sa.Integer(), nullable=False),
        sa.Column("position", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["molecule_id"], ["molecule.id"], ondelete="cascade"),
        sa.ForeignKeyConstraint(["neb_id"], ["neb_record.id"], ondelete="cascade"),
        sa.PrimaryKeyConstraint("neb_id", "molecule_id", "position"),
    )
    op.create_table(
        "neb_optimizations",
        sa.Column("neb_id", sa.Integer(), nullable=False),
        sa.Column("optimization_id", sa.Integer(), nullable=False),
        sa.Column("position", sa.Integer(), nullable=False),
        sa.Column("ts", sa.Boolean(), nullable=False),
        sa.ForeignKeyConstraint(["neb_id"], ["neb_record.id"], ondelete="cascade"),
        sa.ForeignKeyConstraint(
            ["optimization_id"],
            ["optimization_record.id"],
        ),
        sa.PrimaryKeyConstraint("neb_id", "optimization_id", "position", "ts"),
    )
    op.create_table(
        "neb_singlepoints",
        sa.Column("neb_id", sa.Integer(), nullable=False),
        sa.Column("singlepoint_id", sa.Integer(), nullable=False),
        sa.Column("chain_iteration", sa.Integer(), nullable=False),
        sa.Column("position", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["neb_id"], ["neb_record.id"], ondelete="cascade"),
        sa.ForeignKeyConstraint(
            ["singlepoint_id"],
            ["singlepoint_record.id"],
        ),
        sa.PrimaryKeyConstraint("neb_id", "singlepoint_id", "chain_iteration", "position"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    raise RuntimeError("Cannot downgrade")
    # ### end Alembic commands ###



"""change TD/GO dataset specification

Revision ID: f512f2e7ec3d
Revises: 5db09491c69d
Create Date: 2022-07-10 09:45:24.956165

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "f512f2e7ec3d"
down_revision = "5db09491c69d"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    #######################
    # Gridoptimization
    #######################
    op.add_column(
        "gridoptimization_dataset_specification", sa.Column("new_specification_id", sa.Integer(), nullable=True)
    )
    op.create_foreign_key(
        None,
        "gridoptimization_dataset_specification",
        "gridoptimization_specification",
        ["new_specification_id"],
        ["id"],
    )

    op.execute(
        sa.text(
            """
        INSERT INTO gridoptimization_specification (program, optimization_specification_id, keywords)
        SELECT 'gridoptimization', specification_id, '{}'::jsonb
        FROM gridoptimization_dataset_specification
        ON CONFLICT DO NOTHING"""
        )
    )

    op.execute(
        sa.text(
            """
        UPDATE gridoptimization_dataset_specification
        SET new_specification_id = (
            SELECT id FROM gridoptimization_specification
            WHERE optimization_specification_id = specification_id
            AND keywords = '{}'::jsonb
        )"""
        )
    )

    op.drop_column("gridoptimization_dataset_specification", "specification_id")
    op.alter_column(
        "gridoptimization_dataset_specification",
        "new_specification_id",
        nullable=False,
        new_column_name="specification_id",
    )
    op.alter_column(
        "gridoptimization_dataset_entry", "additional_keywords", new_column_name="additional_optimization_keywords"
    )
    op.alter_column(
        "gridoptimization_dataset_entry", "gridoptimization_keywords", new_column_name="additional_keywords"
    )

    op.drop_constraint(
        "gridoptimization_dataset_specificatio_new_specification_id_fkey", "gridoptimization_dataset_specification"
    )
    op.create_index(
        "ix_gridoptimization_dataset_specification_specification_id",
        "gridoptimization_dataset_specification",
        ["specification_id"],
    )
    op.create_foreign_key(
        "gridoptimization_dataset_specification_specification_id_fkey",
        "gridoptimization_dataset_specification",
        "gridoptimization_specification",
        ["specification_id"],
        ["id"],
    )

    #######################
    # Torsiondrive
    #######################
    op.add_column("torsiondrive_dataset_specification", sa.Column("new_specification_id", sa.Integer(), nullable=True))
    op.create_foreign_key(
        None, "torsiondrive_dataset_specification", "torsiondrive_specification", ["new_specification_id"], ["id"]
    )

    op.execute(
        sa.text(
            """
        INSERT INTO torsiondrive_specification (program, optimization_specification_id, keywords)
        SELECT 'torsiondrive', specification_id, '{}'::jsonb
        FROM torsiondrive_dataset_specification
        ON CONFLICT DO NOTHING"""
        )
    )

    op.execute(
        sa.text(
            """
        UPDATE torsiondrive_dataset_specification
        SET new_specification_id = (
            SELECT id FROM torsiondrive_specification
            WHERE optimization_specification_id = specification_id
            AND keywords = '{}'::jsonb
        )"""
        )
    )

    op.drop_column("torsiondrive_dataset_specification", "specification_id")
    op.alter_column(
        "torsiondrive_dataset_specification", "new_specification_id", nullable=False, new_column_name="specification_id"
    )
    op.alter_column(
        "torsiondrive_dataset_entry", "additional_keywords", new_column_name="additional_optimization_keywords"
    )
    op.alter_column("torsiondrive_dataset_entry", "torsiondrive_keywords", new_column_name="additional_keywords")

    op.drop_constraint(
        "torsiondrive_dataset_specification_new_specification_id_fkey", "torsiondrive_dataset_specification"
    )
    op.create_foreign_key(
        "torsiondrive_dataset_specification_specification_id_fkey",
        "torsiondrive_dataset_specification",
        "torsiondrive_specification",
        ["specification_id"],
        ["id"],
    )
    op.create_index(
        "ix_torsiondrive_dataset_specification_specification_id",
        "torsiondrive_dataset_specification",
        ["specification_id"],
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    raise RuntimeError("Cannot downgrade")
    # ### end Alembic commands ###



"""add manybody dataset

Revision ID: 8a41b7dc30af
Revises: c3cbccb2d572
Create Date: 2022-05-15 10:36:52.811760

"""

import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "8a41b7dc30af"
down_revision = "c3cbccb2d572"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "manybody_dataset",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["id"], ["collection.id"], ondelete="cascade"),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "manybody_dataset_entry",
        sa.Column("dataset_id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("comment", sa.String(), nullable=True),
        sa.Column("initial_molecule_id", sa.Integer(), nullable=False),
        sa.Column("additional_keywords", postgresql.JSONB(astext_type=sa.Text()), nullable=False),
        sa.Column("attributes", postgresql.JSONB(astext_type=sa.Text()), nullable=False),
        sa.ForeignKeyConstraint(["dataset_id"], ["manybody_dataset.id"], ondelete="cascade"),
        sa.ForeignKeyConstraint(
            ["initial_molecule_id"],
            ["molecule.id"],
        ),
        sa.PrimaryKeyConstraint("dataset_id", "name"),
    )
    op.create_index("ix_manybody_dataset_entry_dataset_id", "manybody_dataset_entry", ["dataset_id"], unique=False)
    op.create_index(
        "ix_manybody_dataset_entry_initial_molecule_id", "manybody_dataset_entry", ["initial_molecule_id"], unique=False
    )
    op.create_index("ix_manybody_dataset_entry_name", "manybody_dataset_entry", ["name"], unique=False)
    op.create_table(
        "manybody_dataset_specification",
        sa.Column("dataset_id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("description", sa.String(), nullable=True),
        sa.Column("specification_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["dataset_id"], ["manybody_dataset.id"], ondelete="cascade"),
        sa.ForeignKeyConstraint(
            ["specification_id"],
            ["manybody_specification.id"],
        ),
        sa.PrimaryKeyConstraint("dataset_id", "name"),
    )
    op.create_index(
        "ix_manybody_dataset_specification_dataset_id", "manybody_dataset_specification", ["dataset_id"], unique=False
    )
    op.create_index("ix_manybody_dataset_specification_name", "manybody_dataset_specification", ["name"], unique=False)
    op.create_index(
        "ix_manybody_dataset_specification_specification_id",
        "manybody_dataset_specification",
        ["specification_id"],
        unique=False,
    )
    op.create_table(
        "manybody_dataset_record",
        sa.Column("dataset_id", sa.Integer(), nullable=False),
        sa.Column("entry_name", sa.String(), nullable=False),
        sa.Column("specification_name", sa.String(), nullable=False),
        sa.Column("record_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["dataset_id", "entry_name"],
            ["manybody_dataset_entry.dataset_id", "manybody_dataset_entry.name"],
            onupdate="cascade",
            ondelete="cascade",
        ),
        sa.ForeignKeyConstraint(
            ["dataset_id", "specification_name"],
            ["manybody_dataset_specification.dataset_id", "manybody_dataset_specification.name"],
            onupdate="cascade",
            ondelete="cascade",
        ),
        sa.ForeignKeyConstraint(["dataset_id"], ["manybody_dataset.id"], ondelete="cascade"),
        sa.ForeignKeyConstraint(
            ["record_id"],
            ["manybody_record.id"],
        ),
        sa.PrimaryKeyConstraint("dataset_id", "entry_name", "specification_name"),
        sa.UniqueConstraint("dataset_id", "entry_name", "specification_name", name="ux_manybody_dataset_record_unique"),
    )
    op.create_index("ix_manybody_dataset_record_record_id", "manybody_dataset_record", ["record_id"], unique=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("ix_manybody_dataset_record_record_id", table_name="manybody_dataset_record")
    op.drop_table("manybody_dataset_record")
    op.drop_index("ix_manybody_dataset_specification_specification_id", table_name="manybody_dataset_specification")
    op.drop_index("ix_manybody_dataset_specification_name", table_name="manybody_dataset_specification")
    op.drop_index("ix_manybody_dataset_specification_dataset_id", table_name="manybody_dataset_specification")
    op.drop_table("manybody_dataset_specification")
    op.drop_index("ix_manybody_dataset_entry_name", table_name="manybody_dataset_entry")
    op.drop_index("ix_manybody_dataset_entry_initial_molecule_id", table_name="manybody_dataset_entry")
    op.drop_index("ix_manybody_dataset_entry_dataset_id", table_name="manybody_dataset_entry")
    op.drop_table("manybody_dataset_entry")
    op.drop_table("manybody_dataset")
    # ### end Alembic commands ###



"""add triggers for deleting largebinary

Revision ID: 1c46a35bf565
Revises: 63115797e6da
Create Date: 2023-02-07 12:09:40.908204

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "1c46a35bf565"
down_revision = "63115797e6da"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.execute(
        sa.text(
            """
    CREATE OR REPLACE FUNCTION qca_service_subtask_delete_lb()
    RETURNS trigger
    LANGUAGE plpgsql
    AS $function$
    BEGIN
      DELETE FROM largebinary_store
      WHERE largebinary_store.id = OLD.function_kwargs_lb_id OR largebinary_store.id = OLD.results_lb_id;
      RETURN OLD;
    END
    $function$ ;
    """
        )
    )

    op.execute(
        sa.text(
            """
    CREATE OR REPLACE FUNCTION qca_task_queue_delete_lb()
    RETURNS trigger
    LANGUAGE plpgsql
    AS $function$
    BEGIN
      DELETE FROM largebinary_store WHERE largebinary_store.id = OLD.function_kwargs_lb_id;
      RETURN OLD;
    END
    $function$ ;
    """
        )
    )

    op.execute(
        sa.text(
            """
    CREATE TRIGGER qca_service_subtask_delete_lb_tr
    AFTER DELETE ON service_subtask_record
    FOR EACH ROW EXECUTE FUNCTION qca_service_subtask_delete_lb();
    """
        )
    )

    op.execute(
        sa.text(
            """
    CREATE TRIGGER qca_task_queue_delete_lb_tr
    AFTER DELETE ON task_queue
    FOR EACH ROW EXECUTE FUNCTION qca_task_queue_delete_lb();
    """
        )
    )

    op.execute(
        sa.text(
            """
    CREATE FUNCTION qca_largebinary_base_delete() RETURNS TRIGGER AS
    $_$
    BEGIN
      DELETE FROM largebinary_store WHERE largebinary_store.id = OLD.id;
      RETURN OLD;
    END
    $_$ LANGUAGE 'plpgsql';
    """
        )
    )

    # Delete orphan largebinary
    op.execute(
        sa.text(
            """
        WITH anon_1 AS
        (SELECT service_subtask_record.function_kwargs_lb_id AS ref_col FROM service_subtask_record UNION
         SELECT service_subtask_record.results_lb_id AS ref_col FROM service_subtask_record UNION
         SELECT task_queue.function_kwargs_lb_id AS ref_col FROM task_queue)
        DELETE FROM largebinary_store
        WHERE largebinary_store.id NOT IN (SELECT ref_col FROM anon_1 WHERE ref_col IS NOT NULL)
        """
        )
    )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    raise RuntimeError("CANNOT DOWNGRADE")
    # ### end Alembic commands ###



"""add response_bytes to access log

Revision ID: 79604526d271
Revises: 00f7a74dcc37
Create Date: 2021-05-05 09:15:03.601418

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "79604526d271"
down_revision = "00f7a74dcc37"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("access_log", sa.Column("response_bytes", sa.BigInteger(), nullable=True))
    op.add_column("access_log", sa.Column("request_bytes", sa.BigInteger(), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("access_log", "response_bytes")
    op.drop_column("access_log", "request_bytes")
    # ### end Alembic commands ###



"""enable polymorphism for large binary

Revision ID: 63115797e6da
Revises: 377ef3aebfed
Create Date: 2023-02-06 09:21:47.923683

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "63115797e6da"
down_revision = "377ef3aebfed"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("largebinary_store", "record_id")
    op.add_column("largebinary_store", sa.Column("largebinary_type", sa.String(), nullable=True))
    op.execute(sa.text("UPDATE largebinary_store SET largebinary_type = 'generic'"))
    op.alter_column("largebinary_store", "largebinary_type", nullable=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    raise RuntimeError("Cannot downgrade")
    # ### end Alembic commands ###



"""Add reaction service

Revision ID: bba52fa8efbe
Revises: 92ace6197b01
Create Date: 2022-03-10 11:36:13.393303

"""

import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "bba52fa8efbe"
down_revision = "92ace6197b01"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "reaction_record",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("specification_id", sa.Integer(), nullable=False),
        sa.Column("total_energy", postgresql.DOUBLE_PRECISION(), nullable=True),
        sa.ForeignKeyConstraint(["id"], ["base_record.id"], ondelete="cascade"),
        sa.ForeignKeyConstraint(
            ["specification_id"],
            ["qc_specification.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "reaction_stoichiometries",
        sa.Column("reaction_id", sa.Integer(), nullable=False),
        sa.Column("molecule_id", sa.Integer(), nullable=False),
        sa.Column("coefficient", postgresql.DOUBLE_PRECISION(), nullable=False),
        sa.ForeignKeyConstraint(
            ["molecule_id"],
            ["molecule.id"],
        ),
        sa.ForeignKeyConstraint(["reaction_id"], ["reaction_record.id"], ondelete="cascade"),
        sa.PrimaryKeyConstraint("reaction_id", "molecule_id"),
    )
    op.create_table(
        "reaction_components",
        sa.Column("reaction_id", sa.Integer(), nullable=False),
        sa.Column("molecule_id", sa.Integer(), nullable=False),
        sa.Column("singlepoint_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["molecule_id"],
            ["molecule.id"],
        ),
        sa.ForeignKeyConstraint(
            ["reaction_id", "molecule_id"],
            ["reaction_stoichiometries.reaction_id", "reaction_stoichiometries.molecule_id"],
            ondelete="cascade",
        ),
        sa.ForeignKeyConstraint(["reaction_id"], ["reaction_record.id"], ondelete="cascade"),
        sa.ForeignKeyConstraint(
            ["singlepoint_id"],
            ["singlepoint_record.id"],
        ),
        sa.PrimaryKeyConstraint("reaction_id", "molecule_id"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("reaction_components")
    op.drop_table("reaction_stoichiometries")
    op.drop_table("reaction_record")
    # ### end Alembic commands ###



"""refactor service queue

Revision ID: 1ebfc89accb9
Revises: fac12b982c1c
Create Date: 2021-12-16 14:01:11.333690

"""

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "1ebfc89accb9"
down_revision = "fac12b982c1c"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("service_queue", "priority", existing_type=sa.INTEGER(), nullable=False)
    op.alter_column("service_queue", "created_on", nullable=False)
    op.alter_column("service_queue", "record_id", nullable=False)
    op.drop_constraint("service_queue_tasks_record_id_fkey", "service_queue_tasks", type_="foreignkey")
    op.create_foreign_key(None, "service_queue_tasks", "base_record", ["record_id"], ["id"])
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    raise RuntimeError("Cannot downgrade")



"""Renaming of base_result

Revision ID: dd0121101dc2
Revises: 01eb7ec0d803
Create Date: 2021-11-10 12:45:53.662253

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "dd0121101dc2"
down_revision = "01eb7ec0d803"
branch_labels = None
depends_on = None


def upgrade():
    # Drop unused version & hash index columns
    op.drop_column("base_result", "version")
    op.drop_column("base_result", "hash_index")

    ##################################################
    # result-type column is now record type
    # We mostly want what is in the procedure column to be the polymorphic column
    # for inheritance
    op.drop_index("ix_base_result_type", table_name="base_result")
    op.drop_column("base_result", "result_type")
    op.alter_column("base_result", "procedure", new_column_name="record_type", nullable=False)
    op.execute(sa.text("UPDATE base_result SET record_type = 'singlepoint' WHERE record_type = 'single'"))
    op.create_index("ix_base_record_record_type", "base_result", ["record_type"], unique=False)

    ##################################################
    # Rename the status index
    op.execute(sa.text("ALTER INDEX ix_base_result_status RENAME TO ix_base_record_status"))

    ##################################################
    # Now rename the whole table, and the primary key index
    op.rename_table("base_result", "base_record")
    op.execute(sa.text("ALTER SEQUENCE base_result_id_seq RENAME TO base_record_id_seq"))
    op.execute(sa.text("ALTER INDEX base_result_pkey RENAME TO base_record_pkey"))

    ################################################################################
    # Rename base_result/result/procedure to record in task queue
    op.drop_constraint("task_queue_base_result_id_key", "task_queue", type_="unique")
    op.drop_index("ix_task_queue_base_result_id", table_name="task_queue")
    op.alter_column("task_queue", "base_result_id", new_column_name="record_id")
    op.create_unique_constraint("ux_task_queue_record_id", "task_queue", ["record_id"])
    op.execute(
        sa.text("ALTER TABLE task_queue RENAME CONSTRAINT task_queue_base_result_id_fkey TO task_queue_record_id_fkey")
    )

    ################################################################################
    # Same for service queue
    op.drop_constraint("service_queue_procedure_id_key", "service_queue", type_="unique")
    op.alter_column("service_queue", "procedure_id", new_column_name="record_id")
    op.execute(
        sa.text(
            "ALTER TABLE service_queue RENAME CONSTRAINT service_queue_procedure_id_fkey TO service_queue_record_id_fkey"
        )
    )
    op.create_unique_constraint("ux_service_queue_record_id", "service_queue", ["record_id"])

    # And the service queue tasks table
    # There, we have to recreate the primary key
    op.alter_column("service_queue_tasks", "procedure_id", new_column_name="record_id")
    op.execute(
        sa.text(
            "ALTER TABLE service_queue_tasks RENAME CONSTRAINT service_queue_tasks_procedure_id_fkey TO service_queue_tasks_record_id_fkey"
        )
    )

    ##################################################
    # Rename the foreign key constraint to manager name
    # (we drop & recreate because ondelete also changed
    op.drop_constraint("base_result_manager_name_fkey", "base_record", type_="foreignkey")
    op.create_foreign_key("base_record_manager_name_fkey", "base_record", "compute_manager", ["manager_name"], ["name"])
    op.create_index("ix_base_record_manager_name", "base_record", ["manager_name"], unique=False)

    # Create an index on protocols
    # This will be used in later migrations, then the protocols column will be removed
    op.create_index("ix_base_record_protocols", "base_record", ["protocols"], unique=False, postgresql_using="gin")

    ################################################################################
    # Rename columns in server information
    op.alter_column("server_stats_log", "result_count", new_column_name="record_count")

    # (for simplicity, just do a text search and replace inside the JSON fields)
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'base_result', 'base_record')::json"
        )
    )

    op.execute(
        sa.text(
            "UPDATE server_stats_log SET task_queue_status = replace(task_queue_status::text, 'result_type', 'record_type')::json"
        )
    )

    op.execute(
        sa.text(
            "UPDATE server_stats_log SET task_queue_status = replace(task_queue_status::text, 'single', 'singlepoint')::json"
        )
    )

    op.execute(
        sa.text(
            "UPDATE server_stats_log SET service_queue_status = replace(service_queue_status::text, 'result_type', 'record_type')::json"
        )
    )

    # Some columns not nullable
    op.alter_column("base_record", "created_on", nullable=False)
    op.alter_column("base_record", "modified_on", nullable=False)


def downgrade():
    pass



"""NEB spec hashing

Revision ID: 0587bb0220aa
Revises: a3c51b03bc19
Create Date: 2024-12-18 10:18:17.162740

"""

import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import postgresql

from migration_helpers.hashing import hash_dict_1

# revision identifiers, used by Alembic.
revision = "0587bb0220aa"
down_revision = "a3c51b03bc19"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("neb_specification", sa.Column("specification_hash", sa.String(), nullable=True))
    op.add_column("neb_specification", sa.Column("protocols", postgresql.JSONB(astext_type=sa.Text()), nullable=True))

    # Generate hashes for existing specs
    conn = op.get_bind()
    res = conn.execute(sa.text(f"SELECT id, program, keywords FROM neb_specification;"))
    all_spec = res.fetchall()
    for spec_id, program, keywords in all_spec:
        d = {
            "program": program,
            "keywords": keywords,
            "protocols": {},  # Default for now
        }
        h = hash_dict_1(d)
        op.execute(sa.text(f"""UPDATE neb_specification SET specification_hash = '{h}' WHERE id = {spec_id};"""))

    op.execute(sa.text("UPDATE neb_specification SET protocols = '{}'::JSONB"))

    op.alter_column("neb_specification", "protocols", nullable=False)
    op.alter_column("neb_specification", "specification_hash", nullable=False)
    op.drop_constraint("ux_neb_specification_keys", "neb_specification", type_="unique")
    op.create_unique_constraint(
        "ux_neb_specification_keys",
        "neb_specification",
        ["specification_hash", "singlepoint_specification_id", "optimization_specification_id"],
    )
    op.drop_column("neb_specification", "keywords_hash")

    op.create_check_constraint(
        "ck_neb_specification_program_lower", "neb_specification", sa.text("program = LOWER(program)")
    )

    # ### end Alembic commands ###


def downgrade():
    raise RuntimeError("Downgrade not supported.")



"""enable different optimizations in neb

Revision ID: 1638db43303c
Revises: f9d784e816d2
Create Date: 2023-06-05 11:25:35.953639

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "1638db43303c"
down_revision = "f9d784e816d2"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("neb_specification", sa.Column("optimization_specification_id", sa.Integer(), nullable=True))
    op.drop_constraint("ux_neb_specification_keys", "neb_specification", type_="unique")
    op.create_unique_constraint(
        "ux_neb_specification_keys",
        "neb_specification",
        ["program", "singlepoint_specification_id", "optimization_specification_id", "keywords_hash"],
    )
    op.create_index(
        "ix_neb_specification_optimization_specification_id",
        "neb_specification",
        ["optimization_specification_id"],
        unique=False,
    )
    op.create_foreign_key(
        None, "neb_specification", "optimization_specification", ["optimization_specification_id"], ["id"]
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "neb_specification", type_="foreignkey")
    op.drop_index("ix_neb_specification_optimization_specification_id", table_name="neb_specification")
    op.drop_constraint("ux_neb_specification_keys", "neb_specification", type_="unique")
    op.create_unique_constraint(
        "ux_neb_specification_keys", "neb_specification", ["program", "singlepoint_specification_id", "keywords_hash"]
    )
    op.drop_column("neb_specification", "optimization_specification_id")
    # ### end Alembic commands ###



"""fix unique constraints on dataset record items

Revision ID: ca9c57c0df8d
Revises: 8c781020aed4
Create Date: 2023-02-01 10:19:01.065522

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "ca9c57c0df8d"
down_revision = "8c781020aed4"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###

    op.drop_constraint("ux_optimization_dataset_record_unique", "optimization_dataset_record")
    op.create_primary_key(
        "optimization_dataset_record_pkey",
        "optimization_dataset_record",
        ["dataset_id", "entry_name", "specification_name"],
    )

    op.drop_constraint("ux_singlepoint_dataset_record_unique", "singlepoint_dataset_record")
    op.create_primary_key(
        "singlepoint_dataset_record_pkey",
        "singlepoint_dataset_record",
        ["dataset_id", "entry_name", "specification_name"],
    )

    op.drop_constraint("ux_gridoptimization_dataset_record_unique", "gridoptimization_dataset_record")
    op.create_primary_key(
        "gridoptimization_dataset_record_pkey",
        "gridoptimization_dataset_record",
        ["dataset_id", "entry_name", "specification_name"],
    )

    op.drop_constraint("ux_torsiondrive_dataset_record_unique", "torsiondrive_dataset_record")
    op.create_primary_key(
        "torsiondrive_dataset_record_pkey",
        "torsiondrive_dataset_record",
        ["dataset_id", "entry_name", "specification_name"],
    )

    op.drop_constraint("ux_manybody_dataset_record_unique", "manybody_dataset_record")
    op.create_primary_key(
        "manybody_dataset_record_pkey", "manybody_dataset_record", ["dataset_id", "entry_name", "specification_name"]
    )

    op.drop_constraint("ux_reaction_dataset_record_unique", "reaction_dataset_record")
    op.create_primary_key(
        "reaction_dataset_record_pkey", "reaction_dataset_record", ["dataset_id", "entry_name", "specification_name"]
    )

    op.drop_constraint("ux_neb_dataset_record_unique", "neb_dataset_record")
    op.create_primary_key(
        "neb_dataset_record_pkey", "neb_dataset_record", ["dataset_id", "entry_name", "specification_name"]
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    raise RuntimeError("Cannot downgrade")
    # ### end Alembic commands ###



"""remove qcengine from versions

Revision ID: cabe9df168a5
Revises: ae97b389022a
Create Date: 2021-11-04 14:11:40.200284

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "cabe9df168a5"
down_revision = "ae97b389022a"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("versions", "engine_version")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("versions", sa.Column("engine_version", sa.VARCHAR(), autoincrement=False, nullable=True))
    # ### end Alembic commands ###



"""remove keywordset

Revision ID: b9b7b6926b8b
Revises: c1a0b0ee712e
Create Date: 2022-05-21 09:25:14.082795

"""

import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "b9b7b6926b8b"
down_revision = "c1a0b0ee712e"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("qc_specification", sa.Column("keywords", postgresql.JSONB(astext_type=sa.Text()), nullable=True))

    # Migrate the values of the keywords
    op.execute(
        sa.text(
            """UPDATE qc_specification
           SET keywords = keywords.values
           FROM keywords
           WHERE keywords.id = keywords_id
        """
        )
    )

    op.alter_column("qc_specification", "keywords", nullable=False)
    op.drop_index("ix_qc_specification_keywords_id", table_name="qc_specification")
    op.drop_constraint("ux_qc_specification_keys", "qc_specification", type_="unique")
    op.create_unique_constraint(
        "ux_qc_specification_keys",
        "qc_specification",
        ["program", "driver", "method", "basis", "keywords", "protocols"],
    )
    op.create_index("ix_qc_specification_keywords", "qc_specification", ["keywords"], unique=False)
    op.drop_constraint("qc_specification_keywords_id_fkey", "qc_specification", type_="foreignkey")
    op.drop_column("qc_specification", "keywords_id")

    op.drop_table("keywords")
    # ### end Alembic commands ###


def downgrade():
    raise RuntimeError("Cannot downgrade")



"""Add more indicies in association tables

Revision ID: 64cae90ab9fe
Revises: f65aad4015cb
Create Date: 2023-06-27 14:48:06.554515

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "64cae90ab9fe"
down_revision = "f65aad4015cb"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index(
        "ix_gridoptimization_optimization_id", "gridoptimization_optimization", ["optimization_id"], unique=False
    )
    op.create_index("ix_torsiondrive_optimization_id", "torsiondrive_optimization", ["optimization_id"], unique=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("ix_torsiondrive_optimization_id", table_name="torsiondrive_optimization")
    op.drop_index("ix_gridoptimization_optimization_id", table_name="gridoptimization_optimization")
    # ### end Alembic commands ###



"""add indices to neb association tables

Revision ID: 363c647d7840
Revises: 2c48247776d0
Create Date: 2023-05-01 14:41:24.226258

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "363c647d7840"
down_revision = "2c48247776d0"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index("ix_neb_optimizations_optimization_id", "neb_optimizations", ["optimization_id"], unique=False)
    op.create_index("ix_neb_singlepoints_singlepoint_id", "neb_singlepoints", ["singlepoint_id"], unique=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("ix_neb_singlepoints_singlepoint_id", table_name="neb_singlepoints")
    op.drop_index("ix_neb_optimizations_optimization_id", table_name="neb_optimizations")
    # ### end Alembic commands ###



"""Add MOTD

Revision ID: 9c91ede09098
Revises: 301bec558dea
Create Date: 2022-09-08 11:02:28.976308

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "9c91ede09098"
down_revision = "301bec558dea"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "motd",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("motd", sa.String(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("motd")
    # ### end Alembic commands ###



"""Add more indices to results table

Revision ID: 9a6345275dba
Revises: 469ece903d76
Create Date: 2021-09-11 10:36:35.813038

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "9a6345275dba"
down_revision = "469ece903d76"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index("ix_results_basis", "result", ["basis"], unique=False)
    op.create_index("ix_results_driver", "result", ["driver"], unique=False)
    op.create_index("ix_results_keywords", "result", ["keywords"], unique=False)
    op.create_index("ix_results_method", "result", ["method"], unique=False)
    op.create_index("ix_results_molecule", "result", ["molecule"], unique=False)
    op.create_index("ix_results_program", "result", ["program"], unique=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("ix_results_program", table_name="result")
    op.drop_index("ix_results_molecule", table_name="result")
    op.drop_index("ix_results_method", table_name="result")
    op.drop_index("ix_results_keywords", table_name="result")
    op.drop_index("ix_results_driver", table_name="result")
    op.drop_index("ix_results_basis", table_name="result")
    # ### end Alembic commands ###



"""add missing index on optimization_trajectory

Revision ID: 34872c09c1b1
Revises: e8bba86921a5
Create Date: 2022-04-08 14:22:18.851929

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "34872c09c1b1"
down_revision = "e8bba86921a5"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index(
        "ix_optimization_trajectory_singlepoint_id", "optimization_trajectory", ["singlepoint_id"], unique=False
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("ix_optimization_trajectory_singlepoint_id", table_name="optimization_trajectory")
    # ### end Alembic commands ###



"""Fix pkey on alembic_version

Revision ID: 92ace6197b01
Revises: 6137120ef60f
Create Date: 2022-03-08 10:27:22.538234

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "92ace6197b01"
down_revision = "6137120ef60f"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.execute("ALTER TABLE alembic_version DROP CONSTRAINT IF EXISTS alembic_version_pkey")
    op.execute("ALTER TABLE alembic_version DROP CONSTRAINT IF EXISTS alembic_version_pkc")
    op.execute("DROP INDEX IF EXISTS alembic_version_pkey")
    op.execute("DROP INDEX IF EXISTS alembic_version_pkc")

    op.execute("CREATE UNIQUE INDEX alembic_version_pkc ON alembic_version USING btree (version_num)")
    op.execute(
        "ALTER TABLE alembic_version ADD CONSTRAINT alembic_version_pkc PRIMARY KEY USING INDEX alembic_version_pkc"
    )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    raise RuntimeError("Cannot downgrade")
    # ### end Alembic commands ###



"""Add index to compute history

Revision ID: a27802e0d8fe
Revises: a2e52a5e90b6
Create Date: 2023-06-20 18:02:23.182750

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "a27802e0d8fe"
down_revision = "a2e52a5e90b6"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index("ix_record_compute_history_manager_name", "record_compute_history", ["manager_name"], unique=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("ix_record_compute_history_manager_name", table_name="record_compute_history")
    # ### end Alembic commands ###



"""Rename collection table in server log

Revision ID: 301bec558dea
Revises: 48f4d60735cf
Create Date: 2022-09-07 10:05:49.576236

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "301bec558dea"
down_revision = "48f4d60735cf"
branch_labels = None
depends_on = None


def upgrade():
    # Rename entries in the server stats log
    # (for simplicity, just do a text search and replace)
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'collection', 'base_dataset')::json"
        )
    )
    pass


def downgrade():
    pass



"""Add additional_singlepoint_keywords to neb dataset

Revision ID: 3cc95f9dc02c
Revises: 981f69781d65
Create Date: 2022-10-20 11:13:52.166018

"""

import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "3cc95f9dc02c"
down_revision = "981f69781d65"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "neb_dataset_entry",
        sa.Column("additional_singlepoint_keywords", postgresql.JSONB(astext_type=sa.Text()), nullable=True),
    )

    op.execute(sa.text("UPDATE neb_dataset_entry SET additional_singlepoint_keywords = '{}'::jsonb"))

    op.alter_column("neb_dataset_entry", "additional_singlepoint_keywords", nullable=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("neb_dataset_entry", "additional_singlepoint_keywords")
    # ### end Alembic commands ###



"""Add groups to users

Revision ID: 148fef89c2ec
Revises: 6128f0430433
Create Date: 2022-09-26 10:23:26.632594

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "148fef89c2ec"
down_revision = "6128f0430433"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "group",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("groupname", sa.String(), nullable=False),
        sa.Column("description", sa.String(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("groupname", name="ux_group_groupname"),
    )
    op.create_table(
        "user_groups",
        sa.Column("user_id", sa.Integer(), nullable=False),
        sa.Column("group_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["group_id"],
            ["group.id"],
            ondelete="cascade",
        ),
        sa.ForeignKeyConstraint(
            ["user_id"],
            ["user.id"],
            ondelete="cascade",
        ),
        sa.PrimaryKeyConstraint("user_id", "group_id"),
    )

    op.add_column("base_record", sa.Column("owner_user_id", sa.Integer(), nullable=True))
    op.add_column("base_record", sa.Column("owner_group_id", sa.Integer(), nullable=True))
    op.create_index("ix_base_record_owner_user_id", "base_record", ["owner_user_id"], unique=False)
    op.create_index("ix_base_record_owner_group_id", "base_record", ["owner_group_id"], unique=False)
    op.create_foreign_key(None, "base_record", "user", ["owner_user_id"], ["id"])
    op.create_foreign_key(None, "base_record", "group", ["owner_group_id"], ["id"])
    op.create_foreign_key(
        None, "base_record", "user_groups", ["owner_user_id", "owner_group_id"], ["user_id", "group_id"]
    )

    op.add_column("base_dataset", sa.Column("owner_user_id", sa.Integer(), nullable=True))
    op.add_column("base_dataset", sa.Column("owner_group_id", sa.Integer(), nullable=True))
    op.create_index("ix_base_dataset_owner_group_id", "base_dataset", ["owner_group_id"], unique=False)
    op.create_index("ix_base_dataset_owner_user_id", "base_dataset", ["owner_user_id"], unique=False)
    op.create_foreign_key(None, "base_dataset", "user", ["owner_user_id"], ["id"])
    op.create_foreign_key(None, "base_dataset", "group", ["owner_group_id"], ["id"])
    op.create_foreign_key(
        None, "base_dataset", "user_groups", ["owner_user_id", "owner_group_id"], ["user_id", "group_id"]
    )

    op.add_column("record_comment", sa.Column("user_id", sa.Integer(), nullable=True))
    op.create_foreign_key(None, "record_comment", "user", ["user_id"], ["id"])

    op.add_column("internal_jobs", sa.Column("user_id", sa.Integer(), nullable=True))
    op.create_index("ix_internal_jobs_user_id", "internal_jobs", ["user_id"], unique=False)
    op.create_foreign_key(None, "internal_jobs", "user", ["user_id"], ["id"], ondelete="cascade")

    op.add_column("access_log", sa.Column("user_id", sa.Integer(), nullable=True))
    op.create_index("ix_access_log_user_id", "access_log", ["user_id"], unique=False)
    op.create_foreign_key(None, "access_log", "user", ["user_id"], ["id"])

    op.add_column("internal_error_log", sa.Column("user_id", sa.Integer(), nullable=True))
    op.create_index("ix_internal_error_log_user_id", "internal_error_log", ["user_id"], unique=False)
    op.create_foreign_key(None, "internal_error_log", "user", ["user_id"], ["id"])

    op.execute(
        sa.text(
            """
        UPDATE access_log
        SET user_id = u.id
        FROM "user" as u
        WHERE user = u.username;
    """
        )
    )

    op.execute(
        sa.text(
            """
        UPDATE internal_error_log
        SET user_id = u.id
        FROM "user" as u
        WHERE user = u.username;
    """
        )
    )

    op.execute(
        sa.text(
            """
        UPDATE record_comment
        SET user_id = u.id
        FROM "user" as u
        WHERE record_comment.username = u.username;
    """
        )
    )

    op.drop_column("access_log", "user")
    op.drop_column("internal_error_log", "user")
    op.drop_column("internal_jobs", "user")
    op.drop_column("record_comment", "username")

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    raise RuntimeError("Cannot downgrade")
    # ### end Alembic commands ###



"""Clean up some indices

Revision ID: 77baa72171b9
Revises: 64d8c7cc3a18
Create Date: 2022-09-18 09:57:51.222000

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "77baa72171b9"
down_revision = "64d8c7cc3a18"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("ix_access_log_access_type", table_name="access_log")
    op.drop_index("ix_native_file_record_id", table_name="native_file")
    op.drop_index("ix_output_store_history_id", table_name="output_store")
    op.drop_index("ix_record_compute_history_manager_name", table_name="record_compute_history")

    op.drop_index("ix_dataset_type", table_name="base_dataset")
    op.create_index("ix_base_dataset_dataset_type", "base_dataset", ["dataset_type"], unique=False)
    op.drop_constraint("uix_dataset_type_lname", "base_dataset", type_="unique")
    op.create_unique_constraint("ux_base_dataset_dataset_type_lname", "base_dataset", ["dataset_type", "lname"])

    op.create_index("ix_contributed_values_dataset_id", "contributed_values", ["dataset_id"], unique=False)

    # Change some index types
    op.drop_index("ix_access_log_access_date", table_name="access_log")
    op.create_index("ix_access_log_access_date", "access_log", ["access_date"], postgresql_using="brin")

    op.drop_index("ix_internal_error_log_error_date", table_name="internal_error_log")
    op.create_index("ix_internal_error_log_error_date", "internal_error_log", ["error_date"], postgresql_using="brin")

    op.drop_index("ix_server_stats_log_timestamp", table_name="server_stats_log")
    op.create_index("ix_server_stats_log_timestamp", "server_stats_log", ["timestamp"], postgresql_using="brin")

    op.drop_index("ix_molecule_hash", table_name="molecule")
    op.create_index("ix_molecule_molecule_hash", "molecule", ["molecule_hash"], postgresql_using="hash")

    op.drop_index("ix_compute_manager_log_timestamp", table_name="compute_manager")
    op.create_index("ix_compute_manager_log_timestamp", "compute_manager_log", ["timestamp"], postgresql_using="brin")

    op.drop_index("ix_compute_manager_modified_on", table_name="compute_manager_log")
    op.create_index("ix_compute_manager_modified_on", "compute_manager", ["modified_on"], postgresql_using="brin")

    op.drop_index("ix_internal_jobs_added_date", table_name="internal_jobs")
    op.drop_index("ix_internal_jobs_scheduled_date", table_name="internal_jobs")
    op.drop_index("ix_internal_jobs_last_updated", table_name="internal_jobs")
    op.create_index("ix_internal_jobs_added_date", "internal_jobs", ["added_date"], postgresql_using="brin")
    op.create_index("ix_internal_jobs_scheduled_date", "internal_jobs", ["scheduled_date"], postgresql_using="brin")
    op.create_index("ix_internal_jobs_last_updated", "internal_jobs", ["last_updated"], postgresql_using="brin")

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index("ix_record_compute_history_manager_name", "record_compute_history", ["manager_name"], unique=False)
    op.create_index("ix_output_store_history_id", "output_store", ["history_id"], unique=False)
    op.create_index("ix_native_file_record_id", "native_file", ["record_id"], unique=False)
    op.create_index("ix_access_log_access_type", "access_log", ["access_type"], unique=False)

    op.drop_constraint("ux_base_dataset_dataset_type_lname", "base_dataset", type_="unique")
    op.drop_index("ix_base_dataset_dataset_type", table_name="base_dataset")
    op.create_unique_constraint("uix_dataset_type_lname", "base_dataset", ["dataset_type", "lname"])
    op.create_index("ix_dataset_type", "base_dataset", ["dataset_type"], unique=False)

    op.drop_index("ix_contributed_values_dataset_id", table_name="contributed_values")

    op.drop_index("ix_access_log_access_date", table_name="access_log")
    op.create_index("ix_access_log_access_date", "access_log", ["access_date"])

    op.drop_index("ix_internal_error_log_error_date", table_name="internal_error_log")
    op.create_index("ix_internal_error_log_error_date", "internal_error_log", ["error_date"])

    op.drop_index("ix_server_stats_log_timestamp", table_name="server_stats_log")
    op.create_index("ix_server_stats_log_timestamp", "server_stats_log", ["timestamp"])

    op.drop_index("ix_molecule_molecule_hash", table_name="molecule")
    op.create_index("ix_molecule_hash", "molecule", ["molecule_hash"])

    op.drop_index("ix_compute_manager_log_timestamp", table_name="compute_manager")
    op.create_index("ix_compute_manager_log_timestamp", "compute_manager_log", ["timestamp"])

    op.drop_index("ix_compute_manager_modified_on", table_name="compute_manager_log")
    op.create_index("ix_compute_manager_modified_on", "compute_manager", ["modified_on"])

    op.drop_index("ix_internal_jobs_added_date", table_name="internal_jobs")
    op.drop_index("ix_internal_jobs_scheduled_date", table_name="internal_jobs")
    op.drop_index("ix_internal_jobs_last_updated", table_name="internal_jobs")
    op.create_index("ix_internal_jobs_added_date", "internal_jobs", ["added_date"])
    op.create_index("ix_internal_jobs_scheduled_date", "internal_jobs", ["scheduled_date"])
    op.create_index("ix_internal_jobs_last_updated", "internal_jobs", ["last_updated"])

    # ### end Alembic commands ###



"""add is_service to base record

Revision ID: fac12b982c1c
Revises: 7fb1093c30ca
Create Date: 2021-12-13 09:50:43.602929

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "fac12b982c1c"
down_revision = "7fb1093c30ca"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("base_record", sa.Column("is_service", sa.Boolean(), nullable=True))

    # singlepoint and optimizations are never services
    # all existing gridopts and torsiondrives are services
    op.execute(
        sa.text("UPDATE base_record SET is_service = False WHERE record_type IN ('singlepoint', 'optimization')")
    )
    op.execute(
        sa.text("UPDATE base_record SET is_service = True WHERE record_type IN ('torsiondrive', 'gridoptimization')")
    )

    op.alter_column("base_record", "is_service", existing_type=sa.Boolean, nullable=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("base_record", "is_service")
    # ### end Alembic commands ###



"""Migrate task queue orm spec

Revision ID: 11a500398106
Revises: c0fba229aaa8
Create Date: 2023-01-17 20:24:28.657641

"""

import numpy as np
import sqlalchemy as sa
from alembic import op
from sqlalchemy.orm.session import Session
from sqlalchemy.sql import table, column

from qcfractal.db_socket.column_types import MsgpackExt

# revision identifiers, used by Alembic.
revision = "11a500398106"
down_revision = "c0fba229aaa8"
branch_labels = None
depends_on = None


def convert_numpy(obj):
    if isinstance(obj, dict):
        return {k: convert_numpy(v) for k, v in obj.items()}
    elif isinstance(obj, (list, tuple, set)):
        return [convert_numpy(v) for v in obj]
    elif isinstance(obj, np.ndarray):
        if obj.shape:
            return obj.ravel().tolist()
        else:
            return obj.tolist()
    else:
        return obj


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("task_queue", sa.Column("function", sa.String(), nullable=True))
    op.add_column("task_queue", sa.Column("function_kwargs", sa.JSON(), nullable=True))

    task_table = table(
        "task_queue",
        column("id", sa.Integer),
        column("spec", MsgpackExt),
        column("function", sa.String),
        column("function_kwargs", sa.JSON),
    )

    conn = op.get_bind()
    session = Session(conn)

    tasks = session.query(task_table).where(task_table.c.spec.is_not(None)).yield_per(200)

    for task in tasks:
        if task.spec["function"] == "qcengine.compute":
            # Singlepoint
            function_kwargs = {
                "program": task.spec["args"][1],
                "input_data": convert_numpy(task.spec["args"][0]),
            }

        elif task.spec["function"] == "qcengine.compute_procedure":
            function_kwargs = {
                "procedure": task.spec["args"][1],
                "input_data": convert_numpy(task.spec["args"][0]),
            }
        else:
            raise RuntimeError(f"Unknown task function: {task.spec['function']}")

        # Pop some extras if specified
        if "input_data" in function_kwargs and "extras" in function_kwargs["input_data"]:
            function_kwargs["input_data"]["extras"].pop("_qcfractal_tags", None)

        session.execute(
            sa.update(task_table)
            .values(function=task.spec["function"], function_kwargs=function_kwargs)
            .where(task_table.c.id == task.id)
        )

    op.drop_column("task_queue", "spec")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    raise RuntimeError("Cannot downgrade")
    # ### end Alembic commands ###



"""Change status types

Revision ID: 4257f1814d0d
Revises: 88182596f844
Create Date: 2022-11-23 09:32:57.598809

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "4257f1814d0d"
down_revision = "88182596f844"
branch_labels = None
depends_on = None


def upgrade():
    # Add new values to status enum
    # Done in a separate migration because of postgresql limitations
    # ("New enum values must be committed before they can be used")
    op.execute("ALTER TYPE recordstatusenum ADD VALUE 'waiting'")
    op.execute("ALTER TYPE recordstatusenum ADD VALUE 'cancelled'")
    op.execute("ALTER TYPE recordstatusenum ADD VALUE 'deleted'")


def downgrade():
    raise RuntimeError("CANNOT DOWNGRADE")
    pass



"""gridoptimization optimization key to str

Revision ID: e8bba86921a5
Revises: f3ad208b70da
Create Date: 2022-04-07 16:49:34.483749

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "e8bba86921a5"
down_revision = "f3ad208b70da"
branch_labels = None
depends_on = None


def upgrade():
    op.execute("UPDATE gridoptimization_optimization SET key = 'preoptimization' WHERE key = '\"preoptimization\"'")


def downgrade():
    raise RuntimeError("Cannot downgrade")



"""add compute history

Revision ID: 88e6b7d536d0
Revises: dd0121101dc2
Create Date: 2021-11-11 09:10:10.766466

"""

import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import postgresql
from sqlalchemy.orm import declarative_base

Base = declarative_base()

# revision identifiers, used by Alembic.
revision = "88e6b7d536d0"
down_revision = "dd0121101dc2"
branch_labels = None
depends_on = None

status_enum = postgresql.ENUM(
    "complete", "waiting", "running", "error", "cancelled", "deleted", name="recordstatusenum", create_type=False
)


def upgrade():
    # Create the compute history table
    op.create_table(
        "record_compute_history",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("record_id", sa.Integer(), nullable=False),
        sa.Column("status", status_enum, nullable=False),
        sa.Column("manager_name", sa.String(), nullable=True),
        sa.Column("modified_on", sa.DateTime(), nullable=False),
        sa.Column("provenance", sa.JSON(), nullable=True),
        sa.ForeignKeyConstraint(["record_id"], ["base_record.id"], ondelete="CASCADE"),
        sa.ForeignKeyConstraint(["manager_name"], ["compute_manager.name"]),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index("ix_record_compute_history_record_id", "record_compute_history", ["record_id"], unique=False)
    op.create_index("ix_record_compute_history_manager_name", "record_compute_history", ["manager_name"], unique=False)

    # Add columns for record history and output type to the output store
    # nullable for now, will change later in this migration
    op.add_column("output_store", sa.Column("history_id", sa.Integer(), nullable=True))
    op.create_foreign_key(None, "output_store", "record_compute_history", ["history_id"], ["id"], ondelete="CASCADE")
    op.create_index("ix_output_store_history_id", "output_store", ["history_id"])

    output_enum = postgresql.ENUM("stdout", "stderr", "error", name="outputtypeenum")
    output_enum.create(op.get_bind())
    op.add_column("output_store", sa.Column("output_type", output_enum, nullable=True))
    op.create_unique_constraint("ux_output_store_id_type", "output_store", ["history_id", "output_type"])

    #####################################
    # Populate the compute history table
    # Any computations with status = complete or status = error
    op.execute(
        sa.text(
            """
               INSERT INTO record_compute_history (record_id,status,manager_name,modified_on,provenance)
               SELECT id,status,manager_name,modified_on,provenance FROM base_record
               WHERE base_record.status IN ('complete', 'error')
               """
        )
    )

    # Other status with a provenance, manager, output, etc are previously errored calculations
    # that were reset and are now waiting, running, cancelled
    # The base record contains managers that actually attempted to run the calculation. The
    # task queue entry contains what is currently running it
    # modified_on may be incorrect, but it is the best we can do
    op.execute(
        sa.text(
            """
               INSERT INTO record_compute_history (record_id,status,manager_name,modified_on,provenance)
               SELECT id,'error',manager_name,modified_on,provenance FROM base_record
               WHERE base_record.status NOT IN ('complete', 'error')
               AND (base_record.provenance IS NOT NULL
               OR base_record.stdout IS NOT NULL
               OR base_record.stderr IS NOT NULL
               OR base_record.error IS NOT NULL)
               """
        )
    )

    # Copy over the manager from the task queue. Now, we want the base_record manager to
    # reflect the currently-assigned manager
    op.execute(
        sa.text(
            """
               UPDATE base_record
               SET manager_name = task_queue.manager
               FROM task_queue
               WHERE base_record.id = task_queue.record_id
               AND base_record.status = 'running';
               """
        )
    )

    # Now update the output store with foreign keys to history records
    op.execute(
        sa.text(
            """UPDATE output_store SET output_type='stdout', history_id = record_compute_history.id
               FROM base_record
               INNER JOIN record_compute_history ON record_compute_history.record_id = base_record.id
               WHERE base_record.stdout = output_store.id;
               """
        )
    )
    op.execute(
        sa.text(
            """UPDATE output_store SET output_type='stderr', history_id = record_compute_history.id
               FROM base_record
               INNER JOIN record_compute_history ON record_compute_history.record_id = base_record.id
               WHERE base_record.stderr = output_store.id;
               """
        )
    )
    op.execute(
        sa.text(
            """UPDATE output_store SET output_type='error', history_id = record_compute_history.id
               FROM base_record
               INNER JOIN record_compute_history ON record_compute_history.record_id = base_record.id
               WHERE base_record.error = output_store.id;
               """
        )
    )

    # Delete everything from the outputstore that is not linked
    # We do this before dropping the columns on base_record. This will prevent any lost
    # data in output_store in case I screwed up the commands above, and there is data linked to from
    # base_record that hasn't been migrated
    # (since deleting an output_store row that is in-use will cause an exception).
    op.execute(sa.text("DELETE FROM output_store WHERE history_id IS NULL"))

    # Now we can safely delete the columns/indices/constraints in base_record
    op.drop_constraint("base_result_stdout_fkey", "base_record", type_="foreignkey")
    op.drop_constraint("base_result_error_fkey", "base_record", type_="foreignkey")
    op.drop_constraint("base_result_stderr_fkey", "base_record", type_="foreignkey")
    op.drop_index("ix_base_result_stdout", table_name="base_record")
    op.drop_index("ix_base_result_stderr", table_name="base_record")
    op.drop_index("ix_base_result_error", table_name="base_record")
    op.drop_column("base_record", "error")
    op.drop_column("base_record", "stdout")
    op.drop_column("base_record", "stderr")
    op.drop_column("base_record", "provenance")

    # Now we can drop the manager info from the task queue that has been hanging around for a while
    op.drop_index("ix_task_queue_manager", table_name="task_queue")
    op.drop_constraint("task_queue_manager_fkey", "task_queue", type_="foreignkey")
    op.drop_column("task_queue", "manager")

    # Now make stuff not nullable
    op.alter_column("output_store", "history_id", nullable=False)
    op.alter_column("output_store", "output_type", nullable=False)

    # ### end Alembic commands ###


def downgrade():
    raise RuntimeError("Cannot downgrade")



"""Make molecules unique

Revision ID: c0fba229aaa8
Revises: 48d1c43c27e0
Create Date: 2022-12-12 09:39:10.950668

"""

import sqlalchemy as sa
from alembic import op
from sqlalchemy.orm.session import Session

# revision identifiers, used by Alembic.
revision = "c0fba229aaa8"
down_revision = "48d1c43c27e0"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###

    # Find all duplicate molecules
    session = Session(op.get_bind())

    duplicates = session.execute(
        sa.text(
            """
        SELECT molecule_hash, array_agg(id), count(*)
        FROM molecule
        GROUP BY molecule_hash
        HAVING count(*) > 1
    """
        )
    )

    for mol_hash, mol_ids, _ in duplicates:
        mol_ids = sorted(mol_ids)

        # Keep the first id, merge the rest
        keep_id = mol_ids[0]

        dup_ids = f"({','.join(str(x) for x in mol_ids[1:])})"
        op.execute(
            f"UPDATE gridoptimization_dataset_entry  SET initial_molecule_id = {keep_id}  WHERE initial_molecule_id IN {dup_ids}"
        )
        op.execute(
            f"UPDATE gridoptimization_record         SET initial_molecule_id = {keep_id}  WHERE initial_molecule_id IN {dup_ids}"
        )
        op.execute(
            f"UPDATE gridoptimization_record         SET starting_molecule_id = {keep_id} WHERE starting_molecule_id IN {dup_ids}"
        )
        op.execute(
            f"UPDATE manybody_cluster                SET molecule_id = {keep_id}          WHERE molecule_id IN {dup_ids}"
        )
        op.execute(
            f"UPDATE manybody_dataset_entry          SET initial_molecule_id = {keep_id}  WHERE initial_molecule_id IN {dup_ids}"
        )
        op.execute(
            f"UPDATE manybody_record                 SET initial_molecule_id = {keep_id}  WHERE initial_molecule_id IN {dup_ids}"
        )
        op.execute(
            f"UPDATE neb_dataset_molecule            SET molecule_id = {keep_id}          WHERE molecule_id IN {dup_ids}"
        )
        op.execute(
            f"UPDATE neb_initialchain                SET molecule_id = {keep_id}          WHERE molecule_id IN {dup_ids}"
        )
        op.execute(
            f"UPDATE optimization_dataset_entry      SET initial_molecule_id = {keep_id}  WHERE initial_molecule_id IN {dup_ids}"
        )
        op.execute(
            f"UPDATE optimization_record             SET final_molecule_id = {keep_id}    WHERE final_molecule_id IN {dup_ids}"
        )
        op.execute(
            f"UPDATE optimization_record             SET initial_molecule_id = {keep_id}  WHERE initial_molecule_id IN {dup_ids}"
        )
        op.execute(
            f"UPDATE reaction_component              SET molecule_id = {keep_id}          WHERE molecule_id IN {dup_ids}"
        )
        op.execute(
            f"UPDATE reaction_dataset_stoichiometry  SET molecule_id = {keep_id}          WHERE molecule_id IN {dup_ids}"
        )
        op.execute(
            f"UPDATE singlepoint_dataset_entry       SET molecule_id = {keep_id}          WHERE molecule_id IN {dup_ids}"
        )
        op.execute(
            f"UPDATE singlepoint_record              SET molecule_id = {keep_id}          WHERE molecule_id IN {dup_ids}"
        )
        op.execute(
            f"UPDATE torsiondrive_dataset_molecule   SET molecule_id = {keep_id}          WHERE molecule_id IN {dup_ids}"
        )
        op.execute(
            f"UPDATE torsiondrive_initial_molecule   SET molecule_id = {keep_id}          WHERE molecule_id IN {dup_ids}"
        )

        op.execute(f"DELETE FROM molecule WHERE id IN {dup_ids}")

    # Officially no more duplicate molecules
    op.drop_index("ix_molecule_molecule_hash", table_name="molecule")
    op.create_unique_constraint("ux_molecule_molecule_hash", "molecule", ["molecule_hash"])
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint("ux_molecule_molecule_hash", "molecule", type_="unique")
    op.create_index("ix_molecule_molecule_hash", "molecule", ["molecule_hash"], unique=False)
    # ### end Alembic commands ###



"""migrate provenance to not null

Revision ID: 469ece903d76
Revises: 6b07e9a3589d
Create Date: 2021-05-02 09:48:57.061825

"""

from alembic import op
import sqlalchemy as sa
from sqlalchemy.orm.session import Session


# revision identifiers, used by Alembic.
revision = "469ece903d76"
down_revision = "6b07e9a3589d"
branch_labels = None
depends_on = None


def upgrade():
    conn = op.get_bind()
    conn.execute(
        sa.text(
            "UPDATE base_result SET provenance = provenance::jsonb || '{\"creator\":\"\"}' where (provenance->'creator')::text = 'null'"
        )
    )
    conn.execute(
        sa.text(
            "UPDATE base_result SET provenance = provenance::jsonb || '{\"routine\":\"\"}' where (provenance->'routine')::text = 'null'"
        )
    )
    conn.execute(
        sa.text(
            "UPDATE base_result SET provenance = provenance::jsonb || '{\"version\":\"\"}' where (provenance->'version')::text = 'null'"
        )
    )
    conn.execute(
        sa.text(
            "UPDATE molecule SET provenance = provenance::jsonb || '{\"creator\":\"\"}' where (provenance->'creator')::text = 'null'"
        )
    )
    conn.execute(
        sa.text(
            "UPDATE molecule SET provenance = provenance::jsonb || '{\"routine\":\"\"}' where (provenance->'routine')::text = 'null'"
        )
    )
    conn.execute(
        sa.text(
            "UPDATE molecule SET provenance = provenance::jsonb || '{\"version\":\"\"}' where (provenance->'version')::text = 'null'"
        )
    )
    conn.execute(sa.text("UPDATE molecule SET connectivity = null where connectivity::text = '[]'"))
    conn.execute(
        sa.text(
            "UPDATE result SET properties = properties::jsonb - 'mp2_total_correlation_energy' || jsonb_build_object('mp2_correlation_energy', properties->'mp2_total_correlation_energy') WHERE properties::jsonb ? 'mp2_total_correlation_energy'"
        )
    )


def downgrade():
    pass



"""task_queue base_result not null and check lowercase

Revision ID: 88182596f844
Revises: 79604526d271
Create Date: 2021-05-05 09:30:04.702155

"""

from alembic import op
import sqlalchemy as sa
from sqlalchemy.sql import func, column


# revision identifiers, used by Alembic.
revision = "88182596f844"
down_revision = "79604526d271"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("task_queue", "base_result_id", existing_type=sa.INTEGER(), nullable=False)
    op.create_check_constraint("ck_result_program_lower", "result", column("program") == func.lower(column("program")))
    op.create_check_constraint("ck_result_driver_lower", "result", column("driver") == func.lower(column("driver")))
    op.create_check_constraint("ck_result_method_lower", "result", column("method") == func.lower(column("method")))
    op.create_check_constraint("ck_result_basis_lower", "result", column("basis") == func.lower(column("basis")))
    op.create_check_constraint(
        "ck_optimization_procedure_program_lower",
        "optimization_procedure",
        column("program") == func.lower(column("program")),
    )
    op.create_check_constraint(
        "ck_grid_optimization_procedure_program_lower",
        "grid_optimization_procedure",
        column("program") == func.lower(column("program")),
    )
    op.create_check_constraint(
        "ck_torsiondrive_procedure_program_lower",
        "torsiondrive_procedure",
        column("program") == func.lower(column("program")),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("task_queue", "base_result_id", existing_type=sa.INTEGER(), nullable=True)
    op.drop_constraint("ck_result_program_lower", "result")
    op.drop_constraint("ck_result_driver_lower", "result")
    op.drop_constraint("ck_result_method_lower", "result")
    op.drop_constraint("ck_result_basis_lower", "result")
    op.drop_constraint("ck_optimization_procedure_program_lower", "optimization_procedure")
    op.drop_constraint("ck_grid_optimization_procedure_program_lower", "grid_optimization_procedure")
    op.drop_constraint("ck_torsiondrive_procedure_program_lower", "torsiondrive_procedure")
    # ### end Alembic commands ###



"""Not nullable columns on dataset entries

Revision ID: a885e15274b2
Revises: 34872c09c1b1
Create Date: 2022-04-11 10:55:27.980392

"""

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "a885e15274b2"
down_revision = "34872c09c1b1"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.execute(
        sa.text(
            "UPDATE gridoptimization_dataset_entry SET additional_keywords = '{}'::jsonb WHERE additional_keywords IS NULL"
        )
    )
    op.execute(sa.text("UPDATE gridoptimization_dataset_entry SET attributes = '{}'::jsonb WHERE attributes IS NULL"))

    op.execute(
        sa.text(
            "UPDATE singlepoint_dataset_entry SET additional_keywords = '{}'::jsonb WHERE additional_keywords IS NULL"
        )
    )
    op.execute(sa.text("UPDATE singlepoint_dataset_entry SET attributes = '{}'::jsonb WHERE attributes IS NULL"))

    op.execute(
        sa.text(
            "UPDATE optimization_dataset_entry SET additional_keywords = '{}'::jsonb WHERE additional_keywords IS NULL"
        )
    )
    op.execute(sa.text("UPDATE optimization_dataset_entry SET attributes = '{}'::jsonb WHERE attributes IS NULL"))

    op.execute(
        sa.text(
            "UPDATE torsiondrive_dataset_entry SET additional_keywords = '{}'::jsonb WHERE additional_keywords IS NULL"
        )
    )
    op.execute(sa.text("UPDATE torsiondrive_dataset_entry SET attributes = '{}'::jsonb WHERE attributes IS NULL"))

    op.execute(
        sa.text(
            "UPDATE gridoptimization_dataset_entry SET additional_keywords = '{}'::jsonb WHERE additional_keywords::text = 'null'"
        )
    )
    op.execute(
        sa.text("UPDATE gridoptimization_dataset_entry SET attributes = '{}'::jsonb WHERE attributes::text = 'null'")
    )

    op.execute(
        sa.text(
            "UPDATE singlepoint_dataset_entry SET additional_keywords = '{}'::jsonb WHERE additional_keywords::text = 'null'"
        )
    )
    op.execute(sa.text("UPDATE singlepoint_dataset_entry SET attributes = '{}'::jsonb WHERE attributes::text = 'null'"))

    op.execute(
        sa.text(
            "UPDATE optimization_dataset_entry SET additional_keywords = '{}'::jsonb WHERE additional_keywords::text = 'null'"
        )
    )
    op.execute(
        sa.text("UPDATE optimization_dataset_entry SET attributes = '{}'::jsonb WHERE attributes::text = 'null'")
    )

    op.execute(
        sa.text(
            "UPDATE torsiondrive_dataset_entry SET additional_keywords = '{}'::jsonb WHERE additional_keywords::text = 'null'"
        )
    )
    op.execute(
        sa.text("UPDATE torsiondrive_dataset_entry SET attributes = '{}'::jsonb WHERE attributes::text = 'null'")
    )

    op.alter_column(
        "gridoptimization_dataset_entry",
        "additional_keywords",
        existing_type=postgresql.JSONB(astext_type=sa.Text()),
        nullable=False,
    )
    op.alter_column(
        "gridoptimization_dataset_entry",
        "attributes",
        existing_type=postgresql.JSONB(astext_type=sa.Text()),
        nullable=False,
    )
    op.alter_column(
        "optimization_dataset_entry",
        "additional_keywords",
        existing_type=postgresql.JSONB(astext_type=sa.Text()),
        nullable=False,
    )
    op.alter_column(
        "optimization_dataset_entry",
        "attributes",
        existing_type=postgresql.JSONB(astext_type=sa.Text()),
        nullable=False,
    )
    op.alter_column(
        "singlepoint_dataset_entry",
        "additional_keywords",
        existing_type=postgresql.JSONB(astext_type=sa.Text()),
        nullable=False,
    )
    op.alter_column(
        "singlepoint_dataset_entry", "attributes", existing_type=postgresql.JSONB(astext_type=sa.Text()), nullable=False
    )
    op.alter_column(
        "torsiondrive_dataset_entry",
        "additional_keywords",
        existing_type=postgresql.JSONB(astext_type=sa.Text()),
        nullable=False,
    )
    op.alter_column(
        "torsiondrive_dataset_entry",
        "attributes",
        existing_type=postgresql.JSONB(astext_type=sa.Text()),
        nullable=False,
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    raise RuntimeError("Cannot downgrade")



"""rename tables in server logs

Revision ID: 435c1f35227d
Revises: 1c01369d0bae
Create Date: 2022-01-28 15:50:56.612360

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "435c1f35227d"
down_revision = "1c01369d0bae"
branch_labels = None
depends_on = None


def upgrade():
    # Rename entries in the server stats log
    # (for simplicity, just do a text search and replace)
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'queue_manager', 'compute_manager')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'queue_manager_logs', 'compute_manager_logs')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'grid_optimization_association', 'gridoptimization_optimizations')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'grid_optimization_procedure', 'gridoptimization_record')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'kv_store', 'output_store')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'output_store', 'kv_store')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'service_queue_tasks', 'service_dependencies')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'optimization_history', 'torsiondrive_optimizations')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'torsiondrive_procedure', 'torsiondrive_record')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'torsion_init_mol_association', 'torsiondrive_initial_molecules')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'opt_result_association', 'optimization_trajectory')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'optimization_procedure', 'optimization_record')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'base_result', 'base_record')::json"
        )
    )
    op.execute(
        sa.text(
            "UPDATE server_stats_log SET db_table_information = replace(db_table_information::text, 'result', 'singlepoint_record')::json"
        )
    )


def downgrade():
    pass



"""neb subtasks are transient

Revision ID: 377ef3aebfed
Revises: 4122a71f7031
Create Date: 2023-02-01 09:43:23.338150

"""

from alembic import op

# revision identifiers, used by Alembic.
revision = "377ef3aebfed"
down_revision = "4122a71f7031"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("neb_subtasks")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    raise RuntimeError("Cannot downgrade")
    # ### end Alembic commands ###



"""Not nullable columns on base datasets

Revision ID: 97e1d19796c7
Revises: 21e29b68cbd5
Create Date: 2022-06-23 14:51:38.781543

"""

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "97e1d19796c7"
down_revision = "21e29b68cbd5"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.execute(sa.text("UPDATE base_dataset SET tags = '[]'::json WHERE tags IS NULL"))
    op.execute(sa.text("UPDATE base_dataset SET tagline = '' WHERE tagline IS NULL"))
    op.execute(sa.text("UPDATE base_dataset SET description = '' WHERE description IS NULL"))
    op.execute(sa.text("UPDATE base_dataset SET provenance = '{}'::json WHERE provenance IS NULL"))
    op.execute(sa.text("UPDATE base_dataset SET extras = '{}'::json WHERE extras IS NULL"))

    op.alter_column("base_dataset", "tags", existing_type=postgresql.JSON(astext_type=sa.Text()), nullable=False)
    op.alter_column("base_dataset", "tagline", existing_type=sa.VARCHAR(), nullable=False)
    op.alter_column("base_dataset", "description", existing_type=sa.VARCHAR(), nullable=False)
    op.alter_column("base_dataset", "provenance", existing_type=postgresql.JSON(astext_type=sa.Text()), nullable=False)
    op.alter_column("base_dataset", "extras", existing_type=postgresql.JSON(astext_type=sa.Text()), nullable=False)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("base_dataset", "provenance", existing_type=postgresql.JSON(astext_type=sa.Text()), nullable=True)
    op.alter_column("base_dataset", "description", existing_type=sa.VARCHAR(), nullable=True)
    op.alter_column("base_dataset", "tagline", existing_type=sa.VARCHAR(), nullable=True)
    op.alter_column("base_dataset", "tags", existing_type=postgresql.JSON(astext_type=sa.Text()), nullable=True)
    op.alter_column("base_dataset", "extras", existing_type=postgresql.JSON(astext_type=sa.Text()), nullable=True)
    # ### end Alembic commands ###



"""add user and duration to logs

Revision ID: 9e817a4a5cec
Revises: b74cd7a31b69
Create Date: 2021-04-24 10:05:16.592958

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "9e817a4a5cec"
down_revision = "b74cd7a31b69"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("access_log", sa.Column("request_duration", sa.Float(), nullable=True))
    op.add_column("access_log", sa.Column("user", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("access_log", "user")
    op.drop_column("access_log", "request_duration")
    # ### end Alembic commands ###
